"""LUProc.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUProc.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import enum

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------

cProcessWork = 'Main: Идет процесс обработки...'
cProcessStop = 'Main: Процесс обработки остановлен...'
cProcessSetup = 'Setup...'
cProcessAbout = 'About...'
cProcessHelp = 'Help...'
cProcessDeleteAll = 'DeleteAll...'

cProcessBegin = '********* Начало **********************************'
cProcessEnd = '********* Конец ***********************************'

mrOk = True
mrCancel = False

@enum.unique
class TStatApplication(enum.Enum):
    """TStatApplication"""
    saRunning    = enum.auto ()
    saBreak      = enum.auto ()
    saMain       = enum.auto ()
    saDeleteAll  = enum.auto ()
    # saTest     = enum.auto ()
    # saSheduler = enum.auto ()
    # saSetup    = enum.auto ()
    # saAbout    = enum.auto ()
    # saHelp     = enum.auto ()
    # saAction   = enum.auto ()
    # saSend     = enum.auto ()
    # saRefresh  = enum.auto ()
    # saViewLog  = enum.auto ()
    # saFree     = enum.auto ()
    # saStart    = enum.auto ()
    # saStop     = enum.auto ()
    # saAddWidget = enum.auto ()
#endclass
CStatApplication = {
    TStatApplication.saRunning:  'saRunning',
    TStatApplication.saBreak:    'saBreak',
    TStatApplication.saMain:     'saMain',
    TStatApplication.saDeleteAll:     'saDeleteAll'
    # TStatApplication.saTest: 'saTest',
    # TStatApplication.saSheduler: 'saSheduler',
    # TStatApplication.saSetup: 'saSetup',
    # TStatApplication.saAbout: 'saAbout',
    # TStatApplication.saHelp: 'saHelp',
    # TStatApplication.saAction: 'saAction',
    # TStatApplication.saSend: 'saSend',
    # TStatApplication.saRefresh: 'saRefresh',
    # TStatApplication.saViewLog: 'saViewLog',
    # TStatApplication.saFree: 'saFree',
    # TStatApplication.saStart: 'saStart',
    # TStatApplication.saStop: 'saStop',
    # TStatApplication.saAddWidget: 'saAddWidget'
    }
@enum.unique
class TStatWidget(enum.Enum):
    """TStatApplication"""
    swRunning    = enum.auto ()
    swBreak      = enum.auto ()
#endclass
CStatWidget = {
    TStatWidget.swRunning:  'swRunning',
    TStatWidget.swBreak:    'swBreak'
    }

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#---------------------------------------------------------
#
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUQTimer.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUQTimer.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------
from  PySide6 import QtCore

from PySide6.QtCore import (
    QObject, QThread, Signal, Slot, QTimer, QCoreApplication,
    QEventLoop, QTime, QTimer, Slot
    )
from PySide6.QtWidgets import (
    QApplication, QPushButton, QVBoxLayout, QWidget,
    QLCDNumber
    )

#------------------------------------------
# БИБЛИОТЕКА LU
#------------------------------------------
import lyrpy.LULog as LULog

# blocking.py
def wait(milliseconds, /):
    timer = QTimer()
    timer.start(milliseconds)
    wait_for_event(timer.timeout)
def wait_for_event(event, /):
    loop = QEventLoop()
    event.connect(loop.quit)
    loop.exec()


# Signals must inherit QObject
class MySignals(QObject):
    signal_str = Signal(str)
    signal_int = Signal(int)
#endclass

class TQTimer (QTimer):
    """TQTimer"""
    luClassName = 'TQTimer'

    signals = MySignals ()

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__ (self, parent=None):
    #beginfunction
        QTimer.__init__ (self, parent=parent)
        self.__FStopTimer = False
        self.__Fidle = False
        self.__Fparent = parent

        self.FQTimerName: str = ''

        self.interval = 1
        # self.setInterval (1)

        self.signals.signal_str.connect (parent.update_str_field)
        self.signals.signal_int.connect (parent.update_int_field)

        # self.timeout.connect (self.run_CPU)
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__ (self):
        """destructor"""
    #beginfunction
        LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
        #print (s)
    #endfunction

    #--------------------------------------------------
    # @property QTimer
    #--------------------------------------------------
    # getter
    @property
    def QTimer(self) -> QTimer:
    #beginfunction
        return self
    #endfunction

    #--------------------------------------------------
    # start
    #--------------------------------------------------
    def start(self):
        """start - Запуск таймера..."""
    #beginfunction
        s = 'Запуск таймера '+self.FQTimerName+'...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, LULog.DEBUGTEXT, s)
        super ().start ()
        self.__Fidle = True
    #endfunction

    #--------------------------------------------------
    # stop
    #--------------------------------------------------
    def stop(self):
        """stop - Остановить таймер..."""
    #beginfunction
        s = 'Остановка таймера '+self.FQTimerName+'...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, LULog.DEBUGTEXT, s)
        super ().stop ()
    #endfunction

    #--------------------------------------------------
    # __run_TEST
    #--------------------------------------------------
    @QtCore.Slot (str, name = '__run_TEST')
    def __run_TEST(self):
        """__run_TEST..."""
    #beginfunction
        # s = '__run_TEST...'
        # LULog.LoggerTOOLS_AddDebug (s)

        # Do something on the worker thread

        # Emit signals whenever you want
        a = 1 + 1
        self.signals.signal_int.emit (a)
        self.signals.signal_str.emit ("This text comes to Main thread from our Worker thread.")

        # while self.__Fidle:
        #     QCoreApplication.processEvents ()
        # #endwhile

        QCoreApplication.processEvents ()
    #endfunction
#endclass

class DigitalClock(QLCDNumber):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setSegmentStyle(QLCDNumber.Filled)
        self.setDigitCount(8)

        self.timer = QTimer(self)
        self.timer.timeout.connect(self.show_time)
        self.timer.start(1000)

        self.show_time()

        self.setWindowTitle("Digital Clock")
        self.resize(250, 60)

    @Slot (str, name = 'show_time')
    def show_time(self):
        time = QTime.currentTime()
        text = time.toString("hh:mm:ss")

        # Blinking effect
        if (time.second() % 2) == 0:
            text = text.replace(":", " ")

        self.display(text)
#endclass

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#---------------------------------------------------------
#
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUSupport.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2025
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUNumUtils.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------

"""
function Dec2Hex (N: Longint; A: Byte): string;
begin
    Result := IntToHex (N, A);
end;

function D2H (N: Longint; A: Byte): string;
begin
    Result := IntToHex (N, A);
end;

function Hex2Dec (const S: string): Longint;
var
    HexStr: string;
begin
    if Pos ('$', S) = 0 then
        HexStr := '$' + S
    else
        HexStr := S;
    Result := StrToIntDef (HexStr, 0);
end;

function H2D (const S: string): Longint;
begin
    Result := Hex2Dec (S);
end;

function Dec2Numb (N: Longint; A, B: Byte): string;
var
    C: Integer;
{$IFDEF RX_D4}
    Number: Cardinal;
{$ELSE}
    Number: Longint;
{$ENDIF}
begin
    if N = 0 then
        Result := '0'
    else
    begin
{$IFDEF RX_D4}
        Number := Cardinal (N);
{$ELSE}
        Number := N;
{$ENDIF}
        Result := '';
        while Number > 0 do
        begin
            C := Number mod B;
            if C > 9 then
                C := C + 55
            else
                C := C + 48;
            Result := Chr (C) + Result;
            Number := Number div B;
        end;
    end;
    if Result <> '' then
        Result := AddChar ('0', Result, A);
end;

function Numb2Dec (S: string; B: Byte): Longint;
var
    I, P: Longint;
begin
    I := Length (S);
    Result := 0;
    S := UpperCase (S);
    P := 1;
    while (I >= 1) do
    begin
        if S[I] > '@' then
            Result := Result + (Ord(S[I]) - 55) * P
        else
            Result := Result + (Ord(S[I]) - 48) * P;
        Dec (I);
        P := P * B;
    end;
end;

function RomanToInt (const S: string): Longint;
const
    RomanChars = ['C', 'D', 'I', 'L', 'M', 'V', 'X'];
    RomanValues: array ['C' .. 'X'] of Word = (100, 500, 0, 0, 0, 0, 1, 0, 0,
        50, 1000, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 10);
var
    Index, Next: Char;
    I: Integer;
    Negative: Boolean;
begin
    Result := 0;
    I := 0;
    Negative := (Length(S) > 0) and (S[1] = '-');
    if Negative then
        Inc (I);
    while (I < Length(S)) do
    begin
        Inc (I);
        index := UpCase (S[I]);
        if index in RomanChars then
        begin
            if Succ (I) <= Length (S) then
                Next := UpCase (S[I + 1])
            else
                Next := #0;
            if (Next in RomanChars) and (RomanValues[index] < RomanValues[Next])
            then
            begin
                Inc (Result, RomanValues[Next]);
                Dec (Result, RomanValues[index]);
                Inc (I);
            end
            else
                Inc (Result, RomanValues[index]);
        end else begin
            Result := 0;
            Exit;
        end;
    end;
    if Negative then
        Result := - Result;
end;

function IntToRoman (Value: Longint): string;
label A500,
    A400,
    A100,
    A90,
    A50,
    A40,
    A10,
    A9,
    A5,
    A4,
    A1;
begin
    Result := '';
{$IFNDEF WIN32}
    if (Value > MaxInt * 2) then
        Exit;
{$ENDIF}
    while Value >= 1000 do
    begin
        Dec (Value, 1000);
        Result := Result + 'M';
    end;
    if Value < 900 then
        goto A500
    else
    begin
        Dec (Value, 900);
        Result := Result + 'CM';
    end;
    goto A90;
A400:
    if Value < 400 then
        goto A100
    else
    begin
        Dec (Value, 400);
        Result := Result + 'CD';
    end;
    goto A90;
A500:
    if Value < 500 then
        goto A400
    else
    begin
        Dec (Value, 500);
        Result := Result + 'D';
    end;
A100:
    while Value >= 100 do
    begin
        Dec (Value, 100);
        Result := Result + 'C';
    end;
A90:
    if Value < 90 then
        goto A50
    else
    begin
        Dec (Value, 90);
        Result := Result + 'XC';
    end;
    goto A9;
A40:
    if Value < 40 then
        goto A10
    else
    begin
        Dec (Value, 40);
        Result := Result + 'XL';
    end;
    goto A9;
A50:
    if Value < 50 then
        goto A40
    else
    begin
        Dec (Value, 50);
        Result := Result + 'L';
    end;
A10:
    while Value >= 10 do
    begin
        Dec (Value, 10);
        Result := Result + 'X';
    end;
A9:
    if Value < 9 then
        goto A5
    else
    begin
        Result := Result + 'IX';
    end;
    Exit;
A4:
    if Value < 4 then
        goto A1
    else
    begin
        Result := Result + 'IV';
    end;
    Exit;
A5:
    if Value < 5 then
        goto A4
    else
    begin
        Dec (Value, 5);
        Result := Result + 'V';
    end;
    goto A1;
A1:
    while Value >= 1 do
    begin
        Dec (Value);
        Result := Result + 'I';
    end;
end;

function IntToBin (Value: Longint; Digits, Spaces: Integer): string;
begin
    Result := '';
    if Digits > 32 then
        Digits := 32;
    while Digits > 0 do
    begin
        if (Digits mod Spaces) = 0 then
            Result := Result + ' ';
        Dec (Digits);
        Result := Result + IntToStr ((Value shr Digits) and 1);
    end;
end;

"""

#------------------------------------------
def main ():
#beginfunction
    print('main LUNumUtils.py...')
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUThread.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUThread.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import threading
import logging

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКА LU
#------------------------------------------
import lyrpy.LULog as LULog

# class ScheduleThread (threading.Thread):
#     @classmethod
#     def run (cls):
#         while not cease_continuous_run.is_set ():
#             schedule.run_pending ()
#             time.sleep (interval)

# threading.Thread.name
#threading.active_count() количество живых потоков,
#threading.current_thread() текущий поток,
#threading.excepthook() обрабатывает неперехваченные исключения в потоках,
#threading.get_ident() идентификатор текущего потока,
#threading.get_native_id() интегральный идентификатор текущего потока,
#threading.enumerate() список объектов всех живых потоков,
#threading.main_thread() объект основной потока,
#threading.TIMEOUT_MAX максимально значение для тайм-аута блокировки.

#threading.active_count():
#Функция threading.active_count() возвращает количество живых потоков - объектов threading.Thread().
#Возвращенное количество равно длине списка, возвращаемого функцией threading.enumerate().

#threading.get_ident():
#Функция threading.get_ident() возвращает идентификатор текущего потока. Это ненулевое целое число.

#threading.enumerate():
#Функция threading.enumerate() возвращает список объектов threading.Thread() всех живых потоков.

class TThread (threading.Thread):
    """TThread"""
    luClassName = 'TThread'

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__ (self, *args, **kwargs):
    # def __init__ (self, group = None, target = None,
    #                            name = None, args = (),
    #                            kwargs = {}, *, daemon = None):

        """Constructor"""
    #beginfunction
        super ().__init__ (*args, **kwargs)
        # super ().__init__ (group = group, target = target,
        #                        name = name,
        #                         *args,
        #                        **kwargs,
        #                         daemon = daemon)
        #
        self.args = args
        self.kwargs = kwargs
        # print ('args=',args)
        # print ('kwargs=',kwargs)
        self.__FStopThread = False
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__ (self):
        """destructor"""
    #beginfunction
        LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
        #print (s)
    #endfunction

    #--------------------------------------------------
    # @property Thread
    #--------------------------------------------------
    # getter
    @property
    def Thread(self) -> threading.Thread:
    #beginfunction
        return self
    #endfunction

    # #--------------------------------------------------
    # # start
    # #--------------------------------------------------
    # def start(self):
    #     """start - Запуск потока"""
    # #beginfunction
    #     s = 'start - Запуск потока...'
    #     LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
    #     # self.Function ()
    #     super ().start ()
    # #endfunction

    #--------------------------------------------------
    # run
    #--------------------------------------------------
    def run(self):
        """run - Запуск потока"""
    #beginfunction
        s = 'run - Запуск потока...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
        super ().run()
        while not self.__FStopThread:
            # s = 'Выполнение потока...'
            # LULog.LoggerTOOLS_AddDebug (s)
            continue
        #endwhile
    #endfunction

    #--------------------------------------------------
    # StartThread
    #--------------------------------------------------
    def StartThread(self):
        """StartThread"""
    #beginfunction
        s = 'StartThread...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
        self.__FStopThread = False
        self.run()
    #endfunction
    #--------------------------------------------------
    # StopThread
    #--------------------------------------------------
    def StopThread(self):
        """StopThread"""
    #beginfunction
        s = 'StopThread...'
        LULog.LoggerAdd (LULog.LoggerTOOLS, logging.DEBUG, s)
        self.__FStopThread = True
    #endfunction

#endclass

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#---------------------------------------------------------
#
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUNetwork.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2025
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUNetwork.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------

"""
function GetLocalIP: string;
var
    WSAData: TWSAData;
    Host: PHostEnt;
    Buf: array [0 .. 127] of Char;
begin
    if WSAStartup ($101, WSAData) = 0 then
    begin
        if GetHostName (@Buf, 128) = 0 then
        begin
            Host := GetHostByName (@Buf);
            if Host <> nil then
            begin
                Result := iNet_ntoa (PInAddr(Host^.h_addr_list^)^);
            end;
        end;
        WSACleanUP;
    end;
end;
"""

"""
(*
 Пример получения имени пользователя и домена под которым работает
 текущий поток или процесс

 Использовать функцию можно так :
   chDomain:=50;
   chUser :=50;
   GetCurrentUserAndDomain(User,chuser,Domain,chDomain)
 Если вам необходимо получить только имя пользователя - используйте GetUserName
 Данный пример можно использовать и для определения - запущен ли процесс
 системой или пользователем.  Учетной записи Localsystem соответствует
 имя пользователя - SYSTEM и домен NT AUTORITY (лучше проверить на практике)
*)

function GetCurrentUserAndDomain (szUser: PChar; var chUser: DWORD;
    szDomain: PChar; var chDomain: DWORD): Boolean;
var
    hToken: THandle;
    cbBuf: Cardinal;
    ptiUser: PTOKEN_USER;
    snu: SID_NAME_USE;
begin
    Result := False;
   { Получаем маркер доступа текущего потока нашего процесса }
    if not OpenThreadToken (GetCurrentThread(), TOKEN_QUERY, True, hToken) then
    begin
        if GetLastError () <> ERROR_NO_TOKEN then
            Exit;
      { В случее ошибки - получаем маркер доступа нашего процесса. }
        if not OpenProcessToken (GetCurrentProcess(), TOKEN_QUERY, hToken) then
            Exit;
    end;

   { Вывываем GetTokenInformation для получения размера буфера }
    if not GetTokenInformation (hToken, TokenUser, nil, 0, cbBuf) then
        if GetLastError () <> ERROR_INSUFFICIENT_BUFFER then
        begin
            CloseHandle (hToken);
            Exit;
        end;

    if cbBuf = 0 then
        Exit;

   { Выделяем память под буфер }
    GetMem (ptiUser, cbBuf);

   { В случае удачного вызова получим указатель на TOKEN_USER }
    if GetTokenInformation (hToken, TokenUser, ptiUser, cbBuf, cbBuf) then
    begin
      { Ищем имя пользователя и его домен по его SID }
        if LookupAccountSid (nil, ptiUser.User.Sid, szUser, chUser, szDomain,
            chDomain, snu) then
            Result := True;
    end;

   { Освобождаем ресурсы }
    CloseHandle (hToken);
    FreeMem (ptiUser);
end;
"""

"""
function DivideUserName_02 (S: string; var Domain: string;
    var Username: string): Boolean;
var
    i: Integer;
begin
    i := Pos ('\', S);
    if i <> 0 then
    begin
        Domain := Copy (S, 1, i - 1);
        Username := Copy (S, i + 1, Maxint);
        Result := True;
    end else begin
        i := Pos ('@', S);
        if i <> 0 then
        begin
            Username := Copy (S, 1, i - 1);
            Domain := Copy (S, i + 1, Maxint);
         // result := true;
        end else begin
            Domain := '';
            Username := S;
        end;
        Result := False;
    end;
end;
"""

"""
function GetCompName_02: string;
var
    sz: DWORD;
begin
    SetLength (Result, MAX_COMPUTERNAME_LENGTH);
    sz := MAX_COMPUTERNAME_LENGTH + 1;
    GetComputerName (PChar(Result), sz);
    SetLength (Result, sz);
end;
"""

"""
function Error (Res: DWORD): string;
var
    // FErrorBufPtr: Pointer;
    // FNameBufPtr: Pointer;
    FErrorString: string;

    function CheckNetError (Res: DWORD): string;
    var
        S: string;
      // Error: EWin32Error;
    begin
        S := '';
        if (Res <> NERR_Success){ and (Res <> ERROR_MORE_DATA) } then
        begin
            S := SysAndNetErrorMessage (Res);
            S := Format ('Net Error code: %d.'#10'"%s"', [Res, S]);
        end;
        Result := S;
    end;

begin
    FErrorString := CheckNetError (Res);
end;
"""

"""
function GetUserSID_02 (Domain, Username: string; var Sid: PSID;
    var sidType: SID_NAME_USE): Boolean; overload;
var
    pdomain: PChar;
    Size: DWORD;
    refDomainLen: DWORD;
    S: string;
begin
    if Domain <> '' then
        pdomain := PChar (Domain)
    else
        pdomain := nil;
    Size := 4096;
    refDomainLen := 0;
   // result := LookupAccountName(pdomain, pchar(userName), nil, size, nil,refDomainLen,sidType);
    S := Error (GetLastError);
    GetMem (Sid, Size);
    SetLength (Domain, refDomainLen);
    Result := LookupAccountName (pdomain, PChar(Username), Sid, Size,
        @Domain[1], refDomainLen, sidType);
end;
"""

"""
function GetUserSID_02 (Domain, Username: string; var Sid: PSID)
    : Boolean; overload;
var
    ignoreSidType: SID_NAME_USE;
begin
    Result := GetUserSID_02 (Domain, Username, Sid, ignoreSidType);
end;
"""

"""
function GetUserSID_02 (Username: string; var Sid: PSID;
    var sidType: SID_NAME_USE): Boolean; overload;
var
    Domain: string;
begin
    DivideUserName_02 (Username, Domain, Username);
    Result := GetUserSID_02 (Domain, Username, Sid, sidType);
end;
"""

"""
function GetUserSID_02 (Username: string; var Sid: PSID): Boolean; overload;
var
    ignoreSidType: SID_NAME_USE;
begin
    Result := GetUserSID_02 (Username, Sid, ignoreSidType);
end;
"""

"""
function GetUserName_02 (Sid: PSID; out Username: string): Boolean;
var
    Name: string;
    nameLen: DWORD;
    Domain: string;
    domainLen: DWORD;
    sidType: SID_NAME_USE;
begin
    nameLen := 1024;
    SetLength (name, nameLen);
    domainLen := 1024;
    SetLength (Domain, domainLen);
    Result := LookupAccountSid (nil, Sid, @name[1], nameLen, @Domain[1],
        domainLen, sidType);
    if Result then
    begin
        SetLength (name, nameLen);
        SetLength (Domain, domainLen);
        Username := Domain + '\' + name;
    end;
end;
"""

#------------------------------------------
def main ():
#beginfunction
    print('main LUNetwork.py...')
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUObjectsYT.py"""
# -*- coding: UTF-8 -*-
__annotations__ ="""
 =======================================================
 Copyright (c) 2023-2025
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUObjectsYT.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import os
import io
import datetime

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------
from urllib.parse import urlparse
# from pytube import YouTube, request, Playlist, Stream
# import pytube
# import pytube.exceptions

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------
import lyrpy.LULog as LULog
import lyrpy.LUObjects as LUObjects
import lyrpy.LUFile as LUFile
import lyrpy.LUStrUtils as LUStrUtils
import lyrpy.LUThread as LUThread

CYOUTUBE_COM = 'WWW.YOUTUBE.COM'
CYOUTUBE_BE = 'YOUTU.BE'
CYOUTU = 'YOUTU'
CYOUTUBE_PLAYLISTS = 'PLAYLISTS'
CYOUTUBE_PLAYLIST = 'PLAYLIST'
cMaxRes1080p = ('1080p','720p','480p','360p','240p','144p')
cMaxRes720p = ('720p','480p','360p','240p','144p')
cMaxRes480p = ('480p','360p','240p','144p')
cMaxRes360p = ('360p','240p','144p')
cMaxRes240p = ('240p','144p')
cMaxRes144p = ('144p','')

def ONfunction(*args):
#beginfunction
    print('def ONfunction():')
    ...
#endfunction
TONfunction = ONfunction

def progress_func (AStream, AChunk: bytes, Abytes_remaining: int):
#beginfunction
    s = 'progress_func...'
    LULog.LoggerTOOLS_AddDebug (s)
    # if not AStream is None:
    #     LProgressMax = AStream.filesize
    #     LProgressLeft = Abytes_remaining
    #     LProgressValue = LProgressMax - LProgressLeft
    #     if not AChunk is None:
    #         ...
    #     #endif
    #     ...
    # #endif
#endfunction

def complete_func (AStream, AFilePath: str):
#beginfunction
    s = 'complete_func...'
    LULog.LoggerTOOLS_AddDebug (s)
    if not AStream is None:
        LProgressMax = AStream.filesize
        LProgressLeft = 0
        LProgressValue = LProgressMax - LProgressLeft
        # LULog.LoggerTOOLS_AddDebug (f'{LProgressMax}-{LProgressLeft}-{LProgressValue}')

        if not AFilePath is None:
            LFileName = LUStrUtils.PrintableStr (AFilePath)
            LULog.LoggerTOOLS_AddDebug ('Файл ' + LFileName + ' загружен')
        #endif
    #endif
#endfunction

class TYouTubeObject (LUObjects.TObjects):
    """TYouTubeObject"""
    luClassName = "TYouTubeObject"
    
    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__ (self, APathDownload: str):
        """Constructor"""
    #beginfunction
        super ().__init__ ()
        self.__FObjectType: LUObjects.TObjectTypeClass = LUObjects.TObjectTypeClass.otYouTubeObject
        #----------------------------------------------
        self.__FYouTube: YouTube = None
        self.__FStream: pytube.Stream = None
        self.__FURL: str = ''
        self.__FID: datetime = 0
        #----------------------------------------------
        self.__FURLInfo = dict ()
        self.__FStreamInfo = dict ()
        self.__FPlayList: str = ''
        self.__FNumber: int = 0
        self.__FCount: int = 0
        #----------------------------------------------
        self.__FONprogress: TONfunction = self.ONprogressObject
        self.__FONcomplete: TONfunction = self.ONcompleteObject
        #----------------------------------------------
        self.__FYouTubeThread: LUThread.TThread = None
        # self.__FStopYouTubeBoolean: bool = False
        # self.__FStopYouTubeBooleanThread: bool = False
        #----------------------------------------------
        self.FProgressMax: int = 0
        self.FProgressMin: int = 0
        self.FProgressValue: int = 0
        self.FProgressLeft: int = 0
        #----------------------------------------------
        self.FFilechunk: io.BufferedWriter = None
        self.FFileNamechunk = ''
        self.FChunk: bool = True
        self.Fis_paused = False
        self.Fis_cancelled = False
        self.Fskip_existing = False
        #----------------------------------------------
        self.__FPathDownload = APathDownload
        self.__FFilePathDownload = APathDownload
        self.__FFileNamePrefixDownload = ''
        self.__FFileNameDownload = ''
        self.__FFileSizeDownload = 0
        #----------------------------------------------
        self.Clear()
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__(self):
        """destructor"""
    #beginfunction
        super ().__del__()
        LClassName = self.__class__.__name__
        s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
        #print (s)
    #endfunction

    #--------------------------------------------------
    # @property ONprogress
    #--------------------------------------------------
    # getter
    @property
    def ONprogress (self):
    #beginfunction
        return self.__FONprogress
    #endfunction
    @ONprogress.setter
    def ONprogress(self, Value):
    #beginfunction
        self.__FONprogress = Value
    #endfunction

    #--------------------------------------------------
    # @property ONcomplete
    #--------------------------------------------------
    # getter
    @property
    def ONcomplete (self):
    #beginfunction
        return self.__FONcomplete
    #endfunction
    @ONcomplete.setter
    def ONcomplete(self, Value):
    #beginfunction
        self.__FONcomplete = Value
    #endfunction

    #--------------------------------------------------
    # ONprogressObject
    #--------------------------------------------------
    # def show_progress_bar (stream, chunk, bytes_remaining):
    #     prog.update (task, completed = stream.filesize - bytes_remaining)
    #     #this show the bytes_remaining, use rich to display a progress bar
    #--------------------------------------------------
    # def ONprogressObject (self, AStream: pytube.Stream, AChunk: bytes, Abytes_remaining: int):
    def ONprogressObject (self, AStream: Stream, AChunk: bytes, Abytes_remaining: int):
    #beginfunction
        s = 'TYouTubeObject.ONprogressObject...'
        LULog.LoggerTOOLS_AddDebug (s)
        if not AStream is None:
            self.FProgressMax = AStream.filesize
            self.FProgressLeft = Abytes_remaining
            self.FProgressValue = self.FProgressMax - self.FProgressLeft
            if not AChunk is None:
                if not self.FFilechunk is None:
                    self.FFilechunk.write (AChunk)
                #endif
            #endif
        #endif
    #endfunction

    #--------------------------------------------------
    # ONcompleteObject
    #--------------------------------------------------
    # def on_complete (stream, file_path):
    #     prog.remove_task (task)
    #     prog.stop ()
    #     print ('[green] Downloaded ', file_path.split ('/') [-1], '\n')
    #--------------------------------------------------
    # def ONcompleteObject (self, AStream: pytube.Stream, AFilePath: str):
    def ONcompleteObject (self, AStream: Stream, AFilePath: str):
    #beginfunction
        s = 'TYouTubeObject.ONcompleteObject...'
        LULog.LoggerTOOLS_AddDebug (s)

        if not AStream is None:
            self.FProgressMax = AStream.filesize
            self.FProgressLeft = 0
            self.FProgressValue = self.FProgressMax - self.FProgressLeft
            if not AFilePath is None:
                LFileName = LUStrUtils.PrintableStr (AFilePath)
                LULog.LoggerTOOLS_AddDebug ('Файл ' + LFileName + ' загружен')
            #endif
            if not self.__FYouTubeThread is None:
                self.__FYouTubeThread.StopThread()
            #endif
        #endif
    #endfunction

    @staticmethod
    def GetURLInfo (AYouTube) -> {}:
        """GetURLInfo"""
    #beginfunction
        print (f'{AYouTube=}')

        LURLInfo = dict ()

        #Get the video author.
        LURLInfo['author'] = 'AYouTube.author'
        #Get the video description.
        LURLInfo['description'] = 'AYouTube.description'
        #Get the thumbnail url image.
        LURLInfo['thumbnail_url'] = 'AYouTube.thumbnail_url'
        # Заголовок [Get the video title]
        # s = AYouTube.title
        LURLInfo['title'] = 'LUStrUtils.PrintableStr(s)'
        # продолжительность видео [Get the video length in seconds]
        LURLInfo['length'] = 'AYouTube.length'
        LURLInfo['length'] = 0

        # #Get a list of Caption.
        # LURLInfo['caption_tracks'] = AYouTube.caption_tracks
        # #Interface to query caption tracks.
        # LURLInfo['captions'] = AYouTube.captions
        # #Get the video poster’s channel id.
        # LURLInfo['channel_id'] = AYouTube.channel_id
        # #Construct the channel url for the video’s poster from the channel id.
        # LURLInfo['channel_url'] = AYouTube.channel_url
        # #Returns a list of streams if they have been initialized.
        # LURLInfo['fmt_streams'] = AYouTube.fmt_streams
        # #Get the video keywords.
        # LURLInfo['keywords'] = AYouTube.keywords
        # #Get the metadata for the video.
        # LURLInfo['metadata'] = AYouTube.metadata
        # #Get the publish date.
        # LURLInfo['publish_date'] = AYouTube.publish_date
        # #Get the video average rating.
        # LURLInfo['rating'] = AYouTube.rating
        # #Return streamingData from video info.
        # LURLInfo['streaming_data'] = AYouTube.streaming_data
        # #Interface to query both adaptive (DASH) and progressive streams.
        # LURLInfo['streams'] = AYouTube.streams
        # #Parse the raw vid info and return the parsed result.
        # LURLInfo['vid_info'] = AYouTube.vid_info
        # # Количество просмотров [Get the number of the times the video has been viewed]
        # LURLInfo['views'] = AYouTube.views
        return LURLInfo
    #endfunction

    def __SetURLInfo (self):
        """__SetURLInfo"""
    #beginfunction
        print ("""__SetURLInfo""")
        self.__FURLInfo = self.GetURLInfo (self.__FYouTube)
    #endfunction

    @staticmethod
    # def GetStreamInfo (AStream: pytube.Stream) -> {}:
    def GetStreamInfo (AStream: Stream) -> {}:
        """GetStreamInfo"""
    #beginfunction
        LStreamInfo = dict ()
        LFileName = LUStrUtils.PrintableStr(AStream.default_filename)
        LStreamInfo['default_filename'] = LFileName
        # LStreamInfo['title'] = AStream.title
        i = AStream.filesize
        LStreamInfo['filesize'] = i
        # ('1080p', '720p', '480p', '360p', '240p', '144p')
        # LStreamInfo['resolution'] = AStream.resolution
        # 'video'|'audio'
        # LStreamInfo['type'] = AStream.type
        # video/mp4
        # LStreamInfo['mime_type'] = AStream.mime_type
        # itag
        # LStreamInfo['itag'] = AStream.itag
        LStreamInfo['itag'] = 'itag'
        # subtype
        # LStreamInfo['subtype'] = AStream.subtype

        # размер приблизительно
        # LStreamInfo['filesize_approx'] = AStream.filesize_approx
        # размер в Гб
        # LStreamInfo['filesize_gb'] = AStream.filesize_gb
        # размер в Кб
        # LStreamInfo['filesize_kb'] = AStream.filesize_kb
        # размер в Мб
        # LStreamInfo['filesize_mb'] = AStream.filesize_mb
        # url видео плейера
        # LStreamInfo['url'] = AStream.url
        # Get the video/audio codecs from list of codecs.
        # LStreamInfo['parse_codecs'] = AStream.parse_codecs ()
        # True/False - Whether the stream only contains audio.
        # LStreamInfo['includes_audio_track'] = AStream.includes_audio_track
        # True/False - Whether the stream only contains video.
        # LStreamInfo['includes_video_track'] = AStream.includes_video_track
        # #Whether the stream is DASH.
        # LStreamInfo['is_adaptive'] = AStream.is_adaptive
        # #Whether the stream is progressive.
        # LStreamInfo['is_progressive'] = AStream.is_progressive
        # AStream.exists_at_path()
        return LStreamInfo
    #endfunction

    # def __SetStreamInfo (self, AStream: pytube.Stream):
    def __SetStreamInfo (self, AStream: Stream):
        """SetStreamInfo"""
    #beginfunction
        self.__FStreamInfo = self.GetStreamInfo(AStream)
        if len (self.PlayList) > 0:
            self.__FFilePathDownload = os.path.join (self.PathDownload, self.PlayList)
            # if not LUFile.DirectoryExists (self.FilePathDownload):
            #     LUFile.ForceDirectories (self.FilePathDownload)
            # #endif
            self.__FFileNamePrefixDownload = LUStrUtils.AddChar ('0', str (self.Number), 3) + '. '
        else:
            self.__FFilePathDownload = self.PathDownload
            self.__FFileNamePrefixDownload = ''
        #endif
        self.__FFileNameDownload = os.path.join (self.FilePathDownload,
                                                 self.__FFileNamePrefixDownload+self.__FStreamInfo ['default_filename'])
        self.__FFileSizeDownload = self.__FStreamInfo ['filesize']
    #endfunction

    #--------------------------------------------------
    #
    #--------------------------------------------------
    def SetStream(self, AMaxRes: ()):
    #beginfunction
        self.__FStream = None
        for res in AMaxRes:
            LStreams = list ()
            try:
                LStreams = self.__FYouTube.streams.filter (res=res, type='video', file_extension='mp4')
            except BaseException as ERROR:
                s = f'__FYouTube.streams.filter={ERROR} res={res} URL={self.URL}'
                LULog.LoggerTOOLS_AddError(s)
            #endtry
            if len (LStreams) > 0:
                for LStream in LStreams:
                    try:
                        self.__SetStreamInfo (LStream)
                        self.__FStream = LStream
                        break
                    except BaseException as ERROR:
                        s = f'__SetStreamInfo={ERROR} res={res} URL={self.URL}'
                        LULog.LoggerTOOLS_AddError (s)
                    #endtry
                #endfor
                break
            #endif
        #endfor
    #endfunction

    #--------------------------------------------------
    # @property URL
    #--------------------------------------------------
    def SetURL(self, AURL: str, AMaxRes: (), APlayList: str, ANumber: int, ACount: int):
    #beginfunction
        self.__FURL = AURL

        print(f'{AURL=}')
        print(f'{APlayList=}')
        print(f'{ANumber=}')
        print(f'{ACount=}')

        self.__FYouTube: YouTube = YouTube(AURL)
        # self.__FYouTube = Playlist(AURL)
        # self.SetStream(AMaxRes)
        
        self.__SetURLInfo()
        
        self.PlayList = APlayList
        self.Number = ANumber
        self.Count = ACount
    #endfunction

    # getter
    @property
    def URL(self) -> str:
    #beginfunction
        return self.__FURL
    #endfunction

    #--------------------------------------------------
    # @property ID
    #--------------------------------------------------
    # getter
    @property
    def ID(self) -> datetime:
    #beginfunction
        return self.__FID
    #endfunction
    @ID.setter
    def ID(self, Value: datetime):
    #beginfunction
        self.__FID = Value
    #endfunction

    #--------------------------------------------------
    # @property URLInfo
    #--------------------------------------------------
    # getter
    @property
    def URLInfo(self) -> dict:
    #beginfunction
        return self.__FURLInfo
    #endfunction

    #--------------------------------------------------
    # @property StreamInfo
    #--------------------------------------------------
    # getter
    @property
    def StreamInfo(self) -> dict:
    #beginfunction
        return self.__FStreamInfo
    #endfunction

    #--------------------------------------------------
    # @property YOUTUBE
    #--------------------------------------------------
    # getter
    @property
    def YOUTUBE(self) -> YouTube:
    #beginfunction
        return self.__FYouTube
    #endfunction

    #--------------------------------------------------
    # @property PlayList
    #--------------------------------------------------
    # getter
    @property
    def PlayList(self) -> str:
    #beginfunction
        return self.__FPlayList
    #endfunction
    @PlayList.setter
    def PlayList(self, Value: str):
    #beginfunction
        self.__FPlayList = Value
    #endfunction

    #--------------------------------------------------
    # @property FileNameDownload
    #--------------------------------------------------
    # getter
    @property
    def FileNameDownload(self) -> str:
    #beginfunction
        return self.__FFileNameDownload
    #endfunction
    #--------------------------------------------------
    # @property FilePathDownload
    #--------------------------------------------------
    # getter
    @property
    def FilePathDownload(self) -> str:
    #beginfunction
        return self.__FFilePathDownload
    #endfunction
    #--------------------------------------------------
    # @property FileNamePrefixDownload
    #--------------------------------------------------
    # getter
    @property
    def FileNamePrefixDownload(self) -> str:
    #beginfunction
        return self.__FFileNamePrefixDownload
    #endfunction
    #--------------------------------------------------
    # @property FileSizeDownload
    #--------------------------------------------------
    # getter
    @property
    def FileSizeDownload (self) -> int:
    #beginfunction
        return self.__FFileSizeDownload
    #endfunction
    #--------------------------------------------------
    # @property PathDownload
    #--------------------------------------------------
    # getter
    @property
    def PathDownload (self) -> str:
    #beginfunction
        return self.__FPathDownload
    #endfunction
    @PathDownload.setter
    def PathDownload(self, Value: str):
    #beginfunction
        self.__FPathDownload = Value
    #endfunction

    #--------------------------------------------------
    # @property Number
    #--------------------------------------------------
    # getter
    @property
    def Number(self) -> int:
    #beginfunction
        return self.__FNumber
    #endfunction
    @Number.setter
    def Number(self, Value: int):
    #beginfunction
        self.__FNumber = Value
    #endfunction

    #--------------------------------------------------
    # @property Count
    #--------------------------------------------------
    # getter
    @property
    def Count(self) -> int:
    #beginfunction
        return self.__FCount
    #endfunction
    @Count.setter
    def Count(self, Value: int):
    #beginfunction
        self.__FCount = Value
    #endfunction

    #--------------------------------------------------
    # @property ProgressMax
    #--------------------------------------------------
    # getter
    @property
    def ProgressMax(self) -> int:
    #beginfunction
        return self._FProgressMax
    #endfunction
    @ProgressMax.setter
    def ProgressMax(self, Value: int):
    #beginfunction
        self._FProgressMax = Value
    #endfunction

    # #--------------------------------------------------
    # # @property StopYouTubeBoolean
    # #--------------------------------------------------
    # # getter
    # @property
    # def StopYouTubeBoolean(self) -> bool:
    # #beginfunction
    #     return self.__FStopYouTubeBoolean
    # #endfunction
    # @StopYouTubeBoolean.setter
    # def StopYouTubeBoolean(self, Value: bool):
    # #beginfunction
    #     self.__FStopYouTubeBoolean = Value
    # #endfunction

    #--------------------------------------------------
    # @property YouTubeThread
    #--------------------------------------------------
    # getter
    @property
    def YouTubeThread(self):
    #beginfunction
        return self.__FYouTubeThread
    #endfunction
    @YouTubeThread.setter
    def YouTubeThread(self, Value):
    #beginfunction
        self.__FYouTubeThread = Value
    #endfunction

    def Clear (self):
        """Clear"""
    #beginfunction
        # self.__FURL = ''
        # self.ID = 0
        # self.StopYouTubeBoolean = False
        # self.ProgressMax = 0
        # self.YouTubeThread = None
        # self.PlayList = ''
        # self.Number = 0
        ...
    #endfunction

    def __DownloadURL_chunk (self, APATH: str, AFileName:str, AFileSize: int):
        """__DownloadURL_chunk"""
    #beginfunction
        self.FFileNamechunk = AFileName
        LFileSize = AFileSize
        with open(self.FFileNamechunk, 'wb') as self.FFilechunk:
            stream = request.stream(self.__FStream.url) # get an iterable stream
            Ldownloaded = 0
            while True:
                if self.Fis_cancelled:
                    # print('Fis_cancelled...')
                    break
                #endif
                if self.Fis_paused:
                    # print('Fis_paused...')
                    continue
                #endif
                Lchunk = next(stream, None) # get next chunk of video
                if Lchunk:
                    self.__FONprogress (self.__FStream, Lchunk, LFileSize-Ldownloaded)
                    # print(len(Lchunk))
                    Ldownloaded += len(Lchunk)
                else:
                    # no more data
                    self.__FONcomplete (self.__FStream, self.FFileNamechunk)
                    break
                #endif
            #endwhile
        #endwith
    #endfunction

    def DownloadURL (self, ADownload=False, skip_existing = False):
        """DownloadURL"""
    #beginfunction
        if not self.__FONprogress is None:
            self.__FYouTube.register_on_progress_callback(self.__FONprogress)
        if not self.__FONcomplete is None:
            self.__FYouTube.register_on_complete_callback(self.__FONcomplete)

        LPATH = self.FilePathDownload
        if not LUFile.DirectoryExists (LPATH):
            LUFile.ForceDirectories (LPATH)
        #endif
        LFileName = self.FileNameDownload
        LFileSize = self.FileSizeDownload
        if not LUFile.FileExists (LFileName):
            LUFile.FileDelete (LFileName)
        #endif

        if ADownload and len (LFileName) > 0:
            if not self.FChunk:
                try:
                    LFileName = self.__FStream.download (LPATH,
                                                         skip_existing = skip_existing,
                                                         filename_prefix = self.FileNamePrefixDownload)
                # except BaseException as ERROR:
                except Exception as ERROR:
                    s = f'DownloadURL={ERROR}'
                    LULog.LoggerTOOLS_AddError (s)
                #endtry
            else:
                self.__DownloadURL_chunk (LPATH, LFileName, LFileSize)
                if self.Fis_cancelled:
                    if LUFile.FileExists (LFileName):
                        LUFile.FileDelete (LFileName)
                    #endif
                #endif
            #endif
        else:
            N = LFileSize // 4
            i = N
            while i < LFileSize:
                self.__FONprogress(self.__FStream, None, N-i)
                i = i + N
            #endwhile
            self.__FONcomplete (self.__FStream, None)
        #endif
    #endfunction

    def StartYouTubeThread (self, *args, **kwargs):
        """StartYouTubeThread"""
    #beginfunction
        self.__FYouTubeThread = LUThread.TThread(kwargs=kwargs)
        self.__FYouTubeThread.StartThread()
    #endfunction
#endclass

def CheckURLs (AURL: str, AURLs: dict):

    def _CheckPlaylists ():
    #beginfunction
        """
        # ЦИКЛ ОТ i=0 ДО AURLPlaylists.count-1
        """
        ...
    #endfunction

    def _CheckYOUTUBEPlaylist ():
    #beginfunction
        LPlaylist = Playlist (AURL)
        Lvideo_urls = LPlaylist.video_urls
        j = len(LPlaylist.video_urls)
        i = 0
        for url in Lvideo_urls:
            i = i + 1
            LPlaylistTitle = LUStrUtils.DelChars(LPlaylist.title, '/')
            AURLs[url] = {'PlayListName': LPlaylistTitle, 'N': j, 'NN': i}
        #endfor
    #endfunction

#beginfunction
    LURI = urlparse (AURL)
    if LURI.hostname.upper() == CYOUTUBE_COM or LURI.hostname.upper() == CYOUTUBE_BE:
        if CYOUTUBE_PLAYLISTS in LURI.path.upper():
            _CheckPlaylists ()
        else:
            if CYOUTUBE_PLAYLIST in LURI.path.upper():
                _CheckYOUTUBEPlaylist ()
            else:
                if len(LURI.query) > 0:
                    # https: // www.youtube.com / @ nativecode / videos
                    AURLs [AURL] = {'PlayListName': '', 'N': 1, 'NN': 1}
                #endif
            #endif
        #endif
    #endif
#endfunction

#------------------------------------------
#
#------------------------------------------
def DownloadURL (AURL: str, APATH: str, AMaxRes: (), ADownload=False,
                 type='video', file_extension = 'mp4',
                 skip_existing = False, filename_prefix=''):
    """DownloadURL"""
#beginfunction
    LYouTube: YouTube = YouTube (AURL,
                                    on_progress_callback=progress_func,
                                    on_complete_callback=complete_func)
    LURLInfo = TYouTubeObject.GetURLInfo(LYouTube)

    LPATH = APATH

    # все потоки
    LStreams = LYouTube.streams
    # все потоки progressive
    LStreams = LYouTube.streams.filter (progressive = True)
    # все потоки video, mp4, LMaxRes
    LMaxRes = cMaxRes480p
    LMaxRes = cMaxRes1080p
    LMaxRes = AMaxRes

    for res in LMaxRes:
        try:
            LStreams = LYouTube.streams.filter (type=type, file_extension=file_extension, res=res)
        except BaseException as ERROR:
            LStreams = None
            s = f'filter={ERROR}'
            LULog.LoggerTOOLS_AddError (s)
        #endtry

        if not LStreams is None:
            for LStream in LStreams:
                LStreamInfo = TYouTubeObject.GetStreamInfo (LStream)
                LTag = LStreamInfo ['itag']
                # Lresolution = LStreamInfo ['resolution']
                # LULog.LoggerTOOLS_AddDebug (Lresolution)

                Lfilename_prefix = filename_prefix
                LFileName = os.path.join (LPATH, Lfilename_prefix + LStreamInfo ['default_filename'])

                try:
                    if ADownload:
                        if not LStream.exists_at_path (LFileName):
                            s  = LStream.download (APATH, skip_existing=skip_existing, filename_prefix=filename_prefix)
                        else:
                            LULog.LoggerTOOLS_AddDebug ('Файл ' + LFileName + ' существует...')
                            complete_func (LStream, None)
                        #endif
                    else:
                        LFileName = LStreamInfo ['default_filename']
                        AFileSize = int (LStreamInfo ['filesize'])
                        N = AFileSize // 4
                        i = N
                        while i < AFileSize:
                            progress_func (LStream, None, i)
                            i = i + N
                        #endwhile
                        s = f'Видео не загружалось: {res}/{LTag}={LFileName}'
                        LULog.LoggerTOOLS_AddDebug (s)
                        complete_func (LStream, None)
                    #endif
                except BaseException as ERROR:
                    s = f'DownloadURL={ERROR}'
                    LULog.LoggerTOOLS_AddError (s)
                    LULog.LoggerTOOLS_AddError (LFileName)
                #endtry
            #endfor
            # если по фильтру есть хотя бы один поток
            break
        #endif
    #endfor
#endfunction

#------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LULogPySide6.py"""
# -*- coding: UTF-8 -*-
__annotations__ ="""
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LULog.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import os
import sys
import enum
import datetime
import copy
import logging
import logging.config
import yaml
import json
import shutil

import inspect
import traceback

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------
import PySide6

# import pythonjsonlogger
import pythonjsonlogger.jsonlogger

import rich
import rich.console
GConsoleRich = rich.console.Console ()

#------------------------------------------
# БИБЛИОТЕКА LU 
#------------------------------------------
import lyrpy.LUConst as LUConst
import lyrpy.LUFile as LUFile
import lyrpy.LUConsole as LUConsole
import lyrpy.LUDateTime as LUDateTime
import lyrpy.LUos as LUos
import lyrpy.LUParserINI as LUParserINI
import lyrpy.LUDict as LUDict
import lyrpy.LUSupport as LUSupport

# ===========================================================================
# CONST
# ===========================================================================
"""CONST"""
ctlsNOTSET = ' '
ctlsDEBUG = 'D'
ctlsINFO = 'I'
ctlsWARNING = 'W'
ctlsERROR = 'E'
ctlsCRITICAL = 'C'
ctlsBEGIN = '>'
ctlsEND = '<'
ctlsPROCESS = 'P'
ctlsDEBUGTEXT = 'T'
ctlsTEXT = ''

TruncLog = 1
LogPath = ''
Log = 30
LogDir = ''
LogFile = ''

# ДОБАВИТЬ LEVEL
DEBUGTEXT = 11
BEGIN = 21
END = 22
PROCESS = 23
TEXT = 24

# строка формата сообщения
# Cstrfmt_04 = '%(asctime)s %(msecs)03d [%(name)s] %(levelno)02d %(levelname)-8s %(module)s %(message)s'
Cstrfmt_01 = '%(asctime)s [%(name)s] [%(module)-15s] %(levelno)02d %(levelname)-10s %(lineno)04d %(message)s'
Cstrfmt_02 = '%(asctime)s %(name)s %(levelname)-10s %(message)s'
# строка формата времени
Cdatefmt_01 = '%d/%m/%Y %H:%M:%S'
# style
Cstyle_01 = '%'
Cstyle_02 = '{'
Cstyle_03 = '$'
# defaults
Cdefaults = {"ip": '_ip_'}

def AddLevelName():
#beginfunction
    logging.addLevelName(DEBUGTEXT, 'DEBUGTEXT')
    logging.addLevelName(BEGIN, 'BEGIN')
    logging.addLevelName(END, 'END')
    logging.addLevelName(PROCESS, 'PROCESS')
    logging.addLevelName(TEXT, 'TEXT')
#endfunction

CDefaultFileLogINI = 'logging.ini'
CDefaultFileLogINI_CONSOLE = 'logging_CONSOLE.INI'
CDefaultFileLogCONFIG = 'logging.CONFIG'
CDefaultFileLogYAML = 'logging.YAML'

CDefaultFileLog = 'LOGGING.log'
CDefaultFileLogFILEINI = 'LOGGING_FILEINI.log'
CDefaultFileLogFILEINI_json = 'LOGGING_FILEINI_json.log'

CDefaultFileLogFILECONFIG = 'LOGGING_CONFIG.log'
CDefaultFileLogFILECONFIG_json = 'LOGGING_FILECONFIG_json.log'

CDefaultFileLogFILEBASIC = 'LOGGING_BASIC.log'

# ===========================================================================
# type
# ===========================================================================
@enum.unique
class TTypeSETUPLOG (enum.Enum):
    """TTypeSETUPLOG"""
    tslCONFIG = 0
    tslYAML = 1
    tslINI = 2
    @classmethod
    def Empty (cls):
        ...
#endclass

@enum.unique
class TTypeLogString(enum.Enum):
    """TTypeLogString"""
    tlsNOTSET = ctlsNOTSET
    tlsDEBUG = ctlsDEBUG
    tlsINFO = ctlsINFO
    tlsWARNING = ctlsWARNING
    tlsERROR = ctlsERROR
    tlsCRITICAL = ctlsCRITICAL
    tlsBEGIN = ctlsBEGIN
    tlsEND = ctlsEND
    tlsPROCESS = ctlsPROCESS
    tlsDEBUGTEXT = ctlsDEBUGTEXT
    tlsTEXT = ctlsTEXT
    @classmethod
    def Empty(cls):
        ...
#endclass

@enum.unique
class TTypeLogCODE (enum.Enum):
    """TTypeLogCODE"""
    tlcOEM = 0
    tlcANSI = 1
    @classmethod
    def Empty (cls):
        ...
#endclass

@enum.unique
class TLogOutput (enum.Enum):
    loStandard = 0
    loTextFile = 1
    @classmethod
    def Empty (cls):
        ...
#endclass

Cbold = 'bold '
Cblue = 'blue'
Cwhite = 'white'
Cyellow = 'yellow'
Cred = 'red'
Cgreen = 'green'
Con = 'on'

Cbold_blue = Cbold+Cblue
Cbold_white = Cbold+Cwhite
Cbold_yellow = Cbold+Cyellow
Cbold_red = Cbold+Cred
Cbold_red_blue = Cbold+Cred+' on '+Cblue
Cbold_green = Cbold+Cred

COLORS_tls = {
    TTypeLogString.tlsNOTSET: LUConsole.cFG8_BLUE + LUConsole.sEND,
    TTypeLogString.tlsDEBUG: LUConsole.cFG8_BLUE + LUConsole.sEND,
    TTypeLogString.tlsINFO: LUConsole.cFG8_WHITE + LUConsole.sEND,
    TTypeLogString.tlsWARNING: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_YELLOW + LUConsole.sEND,
    TTypeLogString.tlsERROR: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_RED + LUConsole.sEND,
    TTypeLogString.tlsCRITICAL: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_BLACK + ';' + LUConsole.cBG8_RED + LUConsole.sEND,
    TTypeLogString.tlsBEGIN: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_GREEN  + LUConsole.sEND,
    TTypeLogString.tlsEND: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_GREEN + LUConsole.sEND,
    TTypeLogString.tlsPROCESS: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_GREEN + LUConsole.sEND,
    TTypeLogString.tlsDEBUGTEXT: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_BLUE + LUConsole.sEND,
    TTypeLogString.tlsTEXT: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_YELLOW + LUConsole.sEND
}

COLORS_tls_rich = {
    TTypeLogString.tlsNOTSET: Cbold_blue,
    TTypeLogString.tlsDEBUG: Cbold_blue,
    TTypeLogString.tlsINFO: Cbold_white,
    TTypeLogString.tlsWARNING: Cbold_yellow,
    TTypeLogString.tlsERROR: Cbold_red,
    TTypeLogString.tlsCRITICAL: Cbold_red_blue,
    TTypeLogString.tlsBEGIN: Cbold_green,
    TTypeLogString.tlsEND: Cbold_green,
    TTypeLogString.tlsPROCESS: Cbold_green,
    TTypeLogString.tlsDEBUGTEXT: Cbold_blue,
    TTypeLogString.tlsTEXT: Cbold_yellow
}

COLORS = {
    logging.NOTSET: LUConsole.cFG8_BLUE + LUConsole.sEND,
    logging.DEBUG: LUConsole.cFG8_BLUE + LUConsole.sEND,
    logging.INFO: LUConsole.cFG8_WHITE + LUConsole.sEND,
    logging.WARNING: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_YELLOW + LUConsole.sEND,
    logging.ERROR: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_RED + LUConsole.sEND,
    logging.CRITICAL: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_BLACK + ';' + LUConsole.cBG8_RED + LUConsole.sEND,
    BEGIN: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_GREEN + LUConsole.sEND,
    END: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_GREEN + LUConsole.sEND,
    PROCESS: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_GREEN + LUConsole.sEND,
    DEBUGTEXT: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_BLUE + LUConsole.sEND,
    TEXT: LUConsole.cS_BOLD + ';' + LUConsole.cFG8_YELLOW + LUConsole.sEND
}

COLORS_rich = {
    logging.NOTSET: Cbold_blue,
    logging.DEBUG: Cbold_blue,
    logging.INFO: Cbold_white,
    logging.WARNING: Cbold_yellow,
    logging.ERROR: Cbold_red,
    logging.CRITICAL: Cbold_red_blue,
    BEGIN: Cbold_green,
    END: Cbold_green,
    PROCESS: Cbold_green,
    DEBUGTEXT: Cbold_blue,
    TEXT: Cbold_yellow
}

#TLogOutputs = set of TLogOutput;

class TFileMemoLog (object):
    """TFileMemoLog"""
    luClassName = "TFileMemoLog"

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__(self):
        """Constructor"""
    #beginfunction
        super().__init__()
        self.__FCountLogStrings: int = 200
        self.__FFileName: str = ''
        self.__FStandardOut: bool = True
        self.__FLogCODE: TTypeLogCODE = TTypeLogCODE.tlcANSI
        self.__FLogEnabled: bool = True
        self.__FTruncateDays: int = 3
        self.__FLogStringOEM: str = ''
        self.__FLogStringAnsi: str = ''
        self.__FMemoLog = None                        #TMemo
        self.__FLogStrings: list = list()             #TStringList;
        self.__FLogSave: list = list()                #TStringList;
        self.__FLogCODE = LUFile.cDefaultEncoding
        self.__FConsoleRich = rich.console.Console()

        # self.__FLogger: logging.Logger = CreateLoggerFILEINI (CDefaultFileLogINI, 'root')

        self.Clear ()
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__(self):
        """destructor"""
    #beginfunction
        del self.__FLogStrings
        del self.__FLogSave
        # LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        # print (s)
    #endfunction

    def Clear(self):
        """Clear"""
    #beginfunction
        ...
    #endfunction

    #--------------------------------------------------
    # @property LogCODE
    #--------------------------------------------------
    @property
    # getter
    def LogCODE (self) -> int:
    #beginfunction
        return self.__FLogCODE
    #endfunction
    @LogCODE.setter
    def LogCODE (self, Value: int):
    #beginfunction
        self.__FLogCODE = Value
    #endfunction

    #--------------------------------------------------
    # @property CountLogStrings
    #--------------------------------------------------
    @property
    # getter
    def CountLogStrings (self) -> int:
    #beginfunction
        return self.__FCountLogStrings
    #endfunction
    @CountLogStrings.setter
    def CountLogStrings (self, Value: int):
    #beginfunction
        self.__FCountLogStrings = Value
    #endfunction

    #--------------------------------------------------
    # @property LogEnabled
    #--------------------------------------------------
    # getter
    @property
    def LogEnabled (self) -> bool:
    #beginfunction
        return self.__FLogEnabled
    #endfunction
    @LogEnabled.setter
    def LogEnabled (self, Value: bool):
        #beginfunction
        self.__FLogEnabled = Value
    #endfunction

    #--------------------------------------------------
    # @property Filename
    #--------------------------------------------------
    # getter
    @property
    def FileName (self) -> str:
    #beginfunction
        return self.__FFileName
    #endfunction
    @FileName.setter
    def FileName (self, Value: str):
    #beginfunction
        self.__FFileName = Value
        if len(self.__FFileName) > 0 and LUFile.FileExists (self.__FFileName):
            # FMemoLog.Lines.LoadFromFile (self.__FFileName);
            ...
        #endif
    #endfunction

    #--------------------------------------------------
    # @property StandardOut
    #--------------------------------------------------
    # getter
    @property
    def StandardOut (self) -> bool:
    #beginfunction
        return self.__FStandardOut
    #endfunction
    @StandardOut.setter
    def StandardOut (self, Value: bool):
        #beginfunction
        self.__FStandardOut = Value
    #endfunction

    #--------------------------------------------------
    # @property MemoLog
    #--------------------------------------------------
    # getter
    @property
    def MemoLog (self):
    #beginfunction
        return self.__FMemoLog
    #endfunction
    @MemoLog.setter
    def MemoLog (self, Value):
    #beginfunction
        self.__FMemoLog = Value
        #     if FMemoLog <> nil then
        #     begin
        #         with FMemoLog do
        #         begin
        #             Clear;
        #             Align := alClient;
        #             readonly := True;
        #             TabStop := False;
        #             WantReturns := False;
        #             WantTabs := False;
        #             WordWrap := False;
        #             ParentColor := True;
        #             ScrollBars := ssVertical;
        #             ScrollBars := ssBoth;
        #         end;
        #         if (Filename <> '') and FileExists (Filename) then
        #         begin
        #             try
        #                 FMemoLog.Lines.LoadFromFile (Filename);
        #             except
        #             end;
        #         end;
        #     end;
        ...
    #endfunction

    def _SetMemoLog (self, Value):                             #TMemo
        """_SetMemoLog"""
    #beginfunction
        ...
    #endfunction

    @staticmethod
    def _LogDateStr (ATimeOnly: bool) -> str:
        """LogDateStr"""
    #beginfunction
        LToday: datetime.datetime = LUDateTime.Now ()
        if ATimeOnly:
            LResult = ' '*15 + LUDateTime.DateTimeStr (ATimeOnly, LToday, LUDateTime.cFormatDateTimeLog01, True)
        else:
            LResult = LUDateTime.DateTimeStr (ATimeOnly, LToday, LUDateTime.cFormatDateTimeLog01, True)
        #endif
        return LResult
    #endfunction

    def _GetLogSave (self, Filename: str) -> list:  #TStringList
        """_GetLogSave"""
    #beginfunction
        ...
    #endfunction

    def _GetLogSaveCurrent (self) -> list:         #TStringList;
    #beginfunction
        """_GetLogSaveCurrent"""
        LResult = self._GetLogSave (self.__FFileName)
        return LResult
    #endfunction

    def TruncateLog (self):
        """TruncateLog"""
    #beginfunction
        # Filename
        ts: list = list()
        if LUFile.FileExists (self.__FFileName):
            # Открыть для чтения
            LEncoding = LUFile.GetFileEncoding (self.__FFileName)
            LFile = open (self.__FFileName, 'r', encoding = LEncoding)
            try:
                # работа с файлом
                for s in LFile:
                    ts.append (s)
                    #file.next()    возвращает следующую строку файла
                #endfor
            except:
                s = f'TruncateLog: Неправильная кодировка журнала!'
                LoggerTOOLS.error(s)
            finally:
                LFile.close ()
            # TruncateMemo (ts)
            # try
            #     ts.SaveToFile (Filename)
            # except:
            # #endtry
        #endif
        del ts

        # Memo
        ts: list = list()
        if self.__FMemoLog is not None:
            ts.clear()
            # ts.Assign (FMemoLog.Lines)
            # TruncateMemo (ts)
            # FMemoLog.Clear
            # FMemoLog.Lines.Assign (ts)
        #endif
        del ts
    #endfunction

    def _HandlerCONSOLE (self, T: TTypeLogString):
        """_HandlerCONSOLE"""
    #beginfunction

        # if self.FUseColor:

        self.__FLogStrings.clear ()
        self.__FLogStrings.append (self.__FLogStringAnsi)
        for s in self.__FLogStrings:
            if T == TTypeLogString.tlsTEXT:
                _s = s
            else:
                _s = self._LogDateStr (False) + ' ' + str(T.value) + ' ' + s
            if not LUSupport.IsTerminal ():
                LCOLOR = COLORS_tls.get (T)
                if LCOLOR is not None:
                    LFmt = LUConsole.sBEGIN_oct + LCOLOR + _s + LUConsole.sRESET
                else:
                    LFmt = _s
                LUConsole.WriteLN (LFmt)
            else:
                LCOLOR = COLORS_tls_rich.get (T)
                LFmt = _s
                if len(LCOLOR) > 0:
                    LFmt = '[' + LCOLOR + ']' + _s
                else:
                    LFmt = s
                self.__FConsoleRich.print (LFmt)
            #endif
        #endfor
    #endfunction

    def _HandlerFILE (self, T: TTypeLogString):
        """_HandlerFILE"""
    #beginfunction
        s = LUFile.ExpandFileName (self.__FFileName)
        s = LUFile.ExtractFileDir (s)
        if len (s) > 0:
            if not LUFile.DirectoryExists (s):
                LUFile.ForceDirectories (s)
            #endif
        #endif

        self.__FLogStrings.clear ()
        self.__FLogStrings.append (self.__FLogStringAnsi)

        for s in self.__FLogStrings:
            if T == TTypeLogString.tlsTEXT:
                _s = s
            else:
                _s = self._LogDateStr (False) + ' ' + str(T.value) + ' ' + s
            #endif
            try:
                LEncoding = self.__FLogCODE

                # LEncoding = LUFile.GetFileEncoding (self.__FFileName)
                # if LEncoding == '':
                #     LEncoding = LUFile.cDefaultEncoding

                LEncoding = LUFile.cDefaultEncoding
                with open (self.__FFileName, 'a+', encoding = LEncoding) as LFile:

                    # _s = str (s.encode ('utf-8'), 'cp1251')
                    # _s = str (s.encode ('cp1251'), 'cp1251')

                    # _s = str (_s.encode (self.__FLogCODE), self.__FLogCODE)

                    LFile.write (_s + '\n')

                #endwith
            except:
                s = f'_HandlerFILE: Неправильная кодировка журнала!'
                LoggerTOOLS.error (s)
            #endtry
        #endfor
    #endfunction

    def _Execute (self, T: TTypeLogString):
        """_Execute"""
    #beginfunction
        # StandardOut
        if self.__FStandardOut:                 # and isConsole:
            self._HandlerCONSOLE (T)
        #endif
        # Filename
        if self.__FFileName != '':
            self._HandlerFILE (T)
        #endif
        # Memo
        if self.__FMemoLog is not None:
            self.__FLogStrings.clear()
            self.__FLogStrings.append(self.__FLogStringAnsi)
            """
            for s in self.__FLogStrings:
                self.__FMemoLog.add
            #endfor
            """
        #endif
    #endfunction

    #--------------------------------------------------
    #
    #--------------------------------------------------
    def AddLog (self, T: TTypeLogString, Value: str):
        """AddLog"""
    #beginfunction
        self.__FLogStringOEM = Value
        self.__FLogStringAnsi = Value
        if self.LogEnabled:
            self._Execute(T)
        #endif
    #endfunction

    def AddLogFile (self, AFileName: str):
        """AddLogFile"""
    #beginfunction
        if LUFile.FileExists (AFileName):
            LEncoding = LUFile.GetFileEncoding (AFileName)
            if LEncoding == '':
                LEncoding = LUFile.cDefaultEncoding
            #endif
            try:
                # работа с файлом
                with open (AFileName, 'r', encoding = LEncoding) as LFile:
                    for s in LFile:
                        self.AddLog (TTypeLogString.tlsTEXT, s.rstrip('\n'))
                    #endfor
                #endwith
            except:
                self.AddLog (TTypeLogString.tlsERROR, AFileName)
                s = f'AddLogFile: Неправильная кодировка журнала!'
                LoggerTOOLS.error (s)
            #endtry
        #endif
    #endfunction
#endclass

#----------------------------------------------
# TLogging
#----------------------------------------------

#-------------------------------------------------
# TLogRecord(logging.LogRecord):
#-------------------------------------------------
class TLogRecord(logging.LogRecord):
    """TLogRecord"""
    luClassName = "TLogRecord"
    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    #class logging.LogRecord(name, level, pathname, lineno, msg, args, exc_info, func=None, sinfo=None)
    def __init__(self, **kwargs):
        """Constructor"""
    #beginfunction
        logging.LogRecord.__init__(self, **kwargs)
    #endfunction
#endclass

#-------------------------------------------------
# THandler(logging.Handler):
#-------------------------------------------------
class THandler(logging.Handler):
    """THandler"""
    luClassName = "THandler"
    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    #class logging.Handler
    def __init__(self, parent, **kwargs):
    # def __init__ (self, parent):
        """Constructor"""
    #beginfunction
        logging.Handler.__init__(self, **kwargs)
        # super ().__init__ ()

        self.__Fwidget = None
        self.__Fwidget = PySide6.QtWidgets.QPlainTextEdit (parent)
        self.__Fwidget.setReadOnly (True)

    #endfunction

    #--------------------------------------------------
    # @property widget
    #--------------------------------------------------
    # getter
    @property
    def widget (self):
    #beginfunction
        return self.__Fwidget
    #endfunction
    @widget.setter
    def widget (self, Value):
    #beginfunction
        self.__Fwidget = Value
    #endfunction


    def emit(self, record):
        if self.widget is None:
            super (THandler, self).emit (record)
        else:
            msg = self.format (record)
            self.widget.appendPlainText (msg)
        #endif
#endclass

#-------------------------------------------------
# class TStreamHandler(logging.StreamHandler):
#-------------------------------------------------
class TStreamHandler(logging.StreamHandler):
    """TStreamHandler"""
    luClassName = "TStreamHandler"
    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__(self, *args, **kwargs):
        """Constructor"""
    #beginfunction
        logging.StreamHandler.__init__(self, *args, **kwargs)
        self.name = 'CONSOLE'
        self.FAPPGUI = False

        self.__FConsoleRich = rich.console.Console()

        self.__FWidget:PySide6.QtWidgets.QPlainTextEdit = None
        # self.__Fwidget = PySide6.QtWidgets.QPlainTextEdit (parent)
        # self.__Fwidget = PySide6.QtWidgets.QPlainTextEdit ()
        # self.__Fwidget.setReadOnly (True)

    #endfunction

    #--------------------------------------------------
    # @property widget
    #--------------------------------------------------
    # getter
    @property
    def Widget (self):
    #beginfunction
        return self.__FWidget
    #endfunction
    @Widget.setter
    def Widget (self, Value):
    #beginfunction
        self.__FWidget = Value
    #endfunction

    #--------------------------------------------------
    # emit
    #--------------------------------------------------
    def emit(self, record):
        """emit"""
    #beginfunction
        if type(self.formatter) is TFormatter:
            LFormatter: TFormatter = self.formatter
            # widget
            if not self.Widget is None:
                b = LFormatter.FUseColor
                LFormatter.FUseColor = False
                msg = LFormatter.format (record)
                self.Widget.appendPlainText (msg)
                # self.widget.document().end()
                self.Widget.verticalScrollBar().setValue (self.Widget.verticalScrollBar().maximum ())
                LFormatter.FUseColor = b
            #endif

            if LFormatter.FUseColor:
                # self.emit(record)
                try:
                    msg = self.format(record)
                    stream = self.stream
                    # issue 35046: merged two stream.writes into one.
                    if not LUSupport.IsTerminal ():
                        stream.write(msg + self.terminator)
                        self.flush()
                    else:
                        msg = self.format (record)
                        if not self.FAPPGUI:
                            #rich.print (msg)
                            self.__FConsoleRich.print(msg)
                        #endif
                    #endif
                except RecursionError:  # See issue 36272
                    raise
                except Exception:
                    self.handleError(record)
                #endtry
            #endif
        else:
            super(TStreamHandler, self).emit(record)
        #endif
    #endfunction
#endclass

#-------------------------------------------------
# TFilter(logging.Filter):
#-------------------------------------------------
class TFilter(logging.Filter):
    """TFilter"""
    luClassName = "TFilter"
    COLOR = {
        "DEBUG": "BLUE",
        "INFO": "WHITE",
        "WARNING": "YELLOW",
        "ERROR": "RED",
        "CRITICAL": "RED",
        "DEBUGTEXT": "RED",
        "BEGIN": "RED",
        "END": "RED",
        "PROCESS": "RED",
        "TEXT": "RED"
    }
    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    #class logging.Filter (name='')
    def __init__(self, **kwargs):
        """Constructor"""
    #beginfunction
        logging.Filter.__init__(self, **kwargs)
    #endfunction

    def filter(self, record):
    #beginfunction
        record.color = self.COLOR[record.levelname]
        #print(record.color)
        return True
    #endfunction
#endclass

# #-------------------------------------------------
# # TFilter(logging.Filter):
# #-------------------------------------------------
# # Фильтр, который вводит контекстную информацию в журнал.
# # Вместо того, чтобы использовать фактическую контекстуальную информацию, мы
# # просто используем случайные данные в этой демонстрации.
# from random import choice
#
# class TFilter (logging.Filter):
#     """TFilter"""
#     luClassName = "TFilter"
#     USERS = ['jim', 'fred', 'sheila']
#     IPS = ['123.231.231.123', '127.0.0.1', '192.168.0.1']
#     def filter(self, record):
#     #beginfunction
#         record.ip = choice(TFilter.IPS)
#         record.user = choice(TFilter.USERS)
#         return True
#     #endfunction
# #endclass
#
# def Test ():
# #beginfunction
#     levels = (logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR, logging.CRITICAL)
#     logging.basicConfig(level=logging.DEBUG,
#                         format='%(asctime)-15s %(name)-5s %(levelname)-8s IP: %(ip)-15s User: %(user)-8s %(message)s')
#     a1 = logging.getLogger('a.b.c')
#     a2 = logging.getLogger('d.e.f')
#     f = TFilter()
#     a1.addFilter(f)
#     a2.addFilter(f)
#     a1.debug('A debug message')
#     a1.info('An info message with %s', 'some parameters')
#     for x in range(10):
#         lvl = choice(levels)
#         lvlname = logging.getLevelName(lvl)
#         a2.log(lvl, 'A message at %s level with %d %s', lvlname, 2, 'parameters')
# #endfunction

#-------------------------------------------------
# TAdapter(logging.LoggerAdapter):
#-------------------------------------------------
class TAdapter(logging.LoggerAdapter):
    """TAdapter"""
    luClassName = "TAdapter"
    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    #class logging.LoggerAdapter(logger, extra)
    def __init__(self, **kwargs):
        """Constructor"""
    #beginfunction
        # logging.LoggerAdapter.__init__(self, logger = None, extra = None)
        logging.LoggerAdapter.__init__(self, **kwargs)
    #endfunction

    def process(self, msg, kwargs):
        my_context = kwargs.pop('id', self.extra['id'])
        return '[%s] %s' % (my_context, msg), kwargs
#endclass

#-------------------------------------------------
# TFormatter(logging.Formatter):
#-------------------------------------------------
class TFormatter(logging.Formatter):
    """TFormatter"""
    luClassName = "TFormatter"

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__ (self, AUseColor = True, **kwargs):
        """Constructor"""
    #beginfunction
        #class logging.Formatter(fmt=None, datefmt=None, style='%', validate=True, *, defaults=None)
        logging.Formatter.__init__(self, **kwargs)
        self.FUseColor = AUseColor
    #endfunction

    def _SetColor(self, AFmt: str, ALevelNo: int) -> str:
        """_SetColor"""
    #beginfunction
        if self.FUseColor:
            if not LUSupport.IsTerminal ():
                LCOLOR = COLORS.get (ALevelNo)
                LFmt = LUConsole.sBEGIN_oct + LCOLOR + AFmt + LUConsole.sRESET
                return LFmt
            else:
                LCOLOR = COLORS_rich.get (ALevelNo)
                _s = AFmt
                LFmt = _s
                LFmt = '[' + LCOLOR + ']' + _s
                if len(LCOLOR) > 0:
                    LFmt = '[' + LCOLOR + ']' + _s
                else:
                    LFmt = _s
                return LFmt
            #endif
        else:
            return AFmt
        #endif
    #endfunction

    def format(self, record):
        """format"""
    #beginfunction
        # отдельный атрибут
        # LLevelname = record.levelname
        # record.levelname = '_'+LLevelname+'_'

        Ldatefmt = self.datefmt
        if self.FUseColor:
            if record.levelno == TEXT:
                # установить новый fmt
                Lfmt = self._SetColor ('%(message)s', record.levelno)
            else:
                Lfmt = self._SetColor (self._fmt, record.levelno)
            #endif
        else:
            if record.levelno == TEXT:
                # установить новый fmt
                Lfmt = '%(message)s'
            else:
                Lfmt = self._fmt
            #endif
        #endif
        # установить новый fmt
        Lformatter = logging.Formatter (Lfmt, Ldatefmt)
        s = Lformatter.format (record)
        return s
    #endfunction
#endclass

#-------------------------------------------------
# TFormatterJSON(jsonlogger.JsonFormatter):
#-------------------------------------------------
class TFormatterJSON(pythonjsonlogger.jsonlogger.JsonFormatter):
    """TFormatterJSON"""
    luClassName = "TFormatterJSON"

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    #class jsonlogger.JsonFormatter(*args, **kwargs)
    def __init__(self, *args, **kwargs):
        """Constructor"""
    #beginfunction
        super(TFormatterJSON, self).__init__(*args, **kwargs)
        self.json_ensure_ascii = False
        ...
    #endfunction

    def format(self, record):
        """format"""
    #beginfunction
        return super().format(record)
    #endfunction
#endclass

def AddHandlerCONSOLE (ALogger: logging.Logger, ALevel: int, Astrfmt: str, Adatefmt: str,
                    Astyle: str, Adefaults):
    """AddFileHandler"""

#beginfunction
    LHandler = TStreamHandler ()
    LHandler.setLevel (ALevel)
    LHandler.set_name ('CONSOLE')
    LHandler.setStream (sys.stdout)
    LFormater = TFormatter (fmt=Astrfmt, datefmt=Adatefmt,
                                            style=Astyle, validate=True, defaults=Adefaults)
    LHandler.setFormatter (LFormater)
    ALogger.addHandler (LHandler)
#endfunction

def AddHandlerFILE (ALogger: logging.Logger, AFileName: str, ALevel: int, Astrfmt: str, Adatefmt: str,
                    Astyle: str, Adefaults):
    """AddFileHandler"""
#beginfunction
    LHandler = logging.FileHandler (AFileName, mode='a+',
                                    encoding=LUFile.cDefaultEncoding, delay=False, errors=None)
    LHandler.setLevel (ALevel)
    LHandler.set_name ('FILE')
    LFormater = TFormatter (fmt=Astrfmt, datefmt=Adatefmt,
                                    style=Astyle, validate=True, defaults=Adefaults)
    LHandler.setFormatter (LFormater)
    ALogger.addHandler (LHandler)
#endfunction

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# class TLogging (logging.Logger):
# class ColoredLogger(logging.Logger):
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

# ??????????????????????????????????????????????
# class TLogger (logging.getLoggerClass()):
# ??????????????????????????????????????????????

class TLogger (logging.Logger):
    """TLogging"""
    luClassName = "TLogger"

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    def __init__(self, ALogerName: str):
        """Constructor"""
    #beginfunction
        super().__init__(ALogerName)
        self.__FFileName: str = ''
        # Formater
        self.__Fstrfmt = Cstrfmt_01
        self.__Fdatefmt = Cdatefmt_01
        self.__Fstyle = Cstyle_01
        self.__Fdefaults = Cdefaults
        # LEVEL
        self.LEVEL = logging.DEBUG
        # propagate
        self.propagate = True
        self.propagate = False
        AddHandlerCONSOLE (self, self.LEVEL, self.__Fstrfmt, self.__Fdatefmt, self.__Fstyle, self.__Fdefaults)
        self.Clear ()
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__(self):
        """destructor"""
    #beginfunction
        LClassName = self.__class__.__name__
        s = '{} уничтожен'.format(LClassName)
        #print (s)
    #endfunction

    def Clear(self):
        """Clear"""
    #beginfunction
        ...
    #endfunction

    #--------------------------------------------------
    # @property FileName
    #--------------------------------------------------
    # getter
    @property
    def FileName (self) -> str:
    #beginfunction
        return self.__FFileName
    #endfunction
    @FileName.setter
    def FileName (self, Value: str):
    #beginfunction
        self.__FFileName = Value
        if len(self.__FFileName) > 0 and LUFile.FileExists (self.__FFileName):
            ...
        #endif
    #endfunction

    #--------------------------------------------------
    # @property LEVEL
    #--------------------------------------------------
    # getter
    @property
    def LEVEL (self):
    #beginfunction
        return self.level
    #endfunction
    @LEVEL.setter
    def LEVEL (self, Value):
    #beginfunction
        self.setLevel (Value)
    #endfunction

    def AddHandlerCONSOLE(self, ALevel: int):
    #beginfunction
        # AddHandlerCONSOLE (self, ALevel, self.__Fstrfmt, self.__Fdatefmt, self.__Fstyle, self.__Fdefaults)
        # LHandler = logging.StreamHandler ()
        # LHandler.setLevel (ALevel)
        # LHandler.set_name ('CONSOLE')
        # LHandler.setStream (sys.stdout)
        # LFormater = TFormatter (fmt=self.__Fstrfmt, datefmt=self.__Fdatefmt,
        #                                         style=self.__Fstyle, validate=True, defaults=self.__Fdefaults)
        # LHandler.setFormatter (LFormater)
        # self.addHandler (LHandler)
        ...
    #endfunction

    def AddHandlerFILE(self, AFileName: str, ALevel: int):
    #beginfunction
        LHandler = logging.FileHandler (AFileName, mode='a+',
                                        encoding=LUFile.cDefaultEncoding, delay=False, errors=None)
        LHandler.setLevel (ALevel)
        LHandler.set_name ('FILE')
        LFormater = TFormatter (AUseColor = False,
                                fmt=self.__Fstrfmt, datefmt=self.__Fdatefmt,
                                style=self.__Fstyle, validate=True, defaults=self.__Fdefaults)
        LHandler.setFormatter (LFormater)
        self.addHandler (LHandler)
    #endfunction

    def AddHandlerFILE_JSON(self, AFileName: str, ALevel):
    #beginfunction
        LHandler = logging.FileHandler (AFileName, mode='a+',
                                        encoding=LUFile.cDefaultEncoding, delay=False, errors=None)
        LHandler.setLevel (ALevel)
        LHandler.set_name ('FILE_JSON')
        LFormater = TFormatterJSON (AUseColor = False,
                                    fmt=self.__Fstrfmt, datefmt=self.__Fdatefmt,
                                    style=self.__Fstyle, validate=True, defaults=self.__Fdefaults)
        LFormater.json_ensure_ascii = False
        LHandler.setFormatter (LFormater)
        self.addHandler (LHandler)
    #endfunction
#endclass

"""
# logger
logger = logging.getLogger(__name__)
    
    # LEVEL
    NOTSET
    DEBUG - уровень отладочной информации, зачастую помогает при разработке приложения на машине программиста.
    INFO - уровень вспомогательной информации о ходе работы приложения/скрипта.
    WARNING - уровень предупреждения. Например, мы можем предупреждать о том, что та или иная функция будет удалена в будущих версиях вашего приложения.
    ERROR - с таким уровнем разработчики пишут логи с ошибками, например, о том, что внешний сервис недоступен.
    CRITICAL - уровень сообщений после которых работа приложения продолжаться не может.
    
    # УСТАНОВИТЬ LEVEL
    logger.setLevel(logging.DEBUG)
    logger.setLevel(logging.INFO)
    
    # ЗАПИСЬ в logger
    logger.debug('debug')
    logger.info('info')
    logger.warning('warning')
    logger.error('error')
    logger.exception('error')
    logger.critical('critical')

    # handler
    Задача класса Handler и его потомков обрабатывать запись сообщений/логов. Т.е. Handler отвечает за то куда будут записаны сообщения. В базовом наборе logging предоставляет ряд готовых классов-обработчиков:
    SteamHandler - запись в поток, например, stdout или stderr.
        handler = StreamHandler(stream=sys.stdout)
    FileHandler - запись в файл, класс имеет множество производных классов с различной функциональностью
        ротация файлов логов по размеру, времени и т.д.)
        handler = StreamHandler(stream=)
    BaseRotatingHandler
        handler = BaseRotatingHandler(filename, mode, encoding=None, delay=False, errors=None
    RotatingFileHandler
        handler = RotatingFileHandler(filename, mode='a', maxBytes=0, backupCount=0, encoding=None,
            delay=False, errors=None
    TimedRotatingFileHandler
        handler = TimedRotatingFileHandler(filename, when='h', interval=1, backupCount=0, encoding=None,
            delay=False, utc=False, atTime=None, errors=None) 
        
    SocketHandler - запись сообщений в сокет по TCP
        handler = SocketHandler(host, port)
    DatagramHandler - запись сообщений в сокет по UDP
        handler = DatagramHandler(host, port)
    SysLogHandler - запись в syslog
        handler = SysLogHandler(address=('localhost', SYSLOG_UDP_PORT), facility=LOG_USER, socktype=socket.SOCK_DGRAM)
    HTTPHandler - запись по HTTP
        handler = HTTPHandler(host, url, method='GET', secure=False, credentials=None, context=None)
    NullHandler = NullHandler
        handler = StreamHandler(stream=)
    WatchedFileHandler
        handler = WatchedFileHandler(filename, mode='a', encoding=None, delay=False, errors=None)
    NTEventLogHandler
        handler = NTEventLogHandler(appname, dllname=None, logtype='Application')
    SMTPHandler
        handler = SMTPHandler(mailhost, fromaddr, toaddrs, subject, credentials=None, secure=None, timeout=1.0)
    MemoryHandler
        handler = BufferingHandler(capacity)¶
    QueueHandler
        handler = QueueHandler(queue)
    QueueListener
        handler = QueueListener(queue, *handlers, respect_handler_level=False)¶
        
    # ДОБАВИТЬ handler
    logger.addHandler(handler)

    # Formatter
    #class logging.Formatter(fmt=None, datefmt=None, style='%', validate=True, *, defaults=None)
    formster = logging.Formatter (fmt=self.strfmt_03, datefmt=self.datefmt_02, style = '%', validate = True,
        defaults = {"ip": None} )
    handler.setFormatter(Formatter(fmt='[%(asctime)s: %(levelname)s] %(message)s'))

    Параметр defaults может быть словарем со значениями по умолчанию для использования в настраиваемых полях.
    Например: logging.Formatter('%(ip)s %(message)s', defaults={"ip": None})

    # Filter
    def filter_python(record: LogRecord) -> bool:
        return record.getMessage().find('python') != -1
    logger.addFilter(filter_python)

    # LoggerAdapter
    class CustomLoggerAdapter(LoggerAdapter):
    def process(self, msg, kwargs):
        return f'{msg} from {self.extra["username"]}', kwargs
    
    logger2 = logging.getLogger('adapter')
    logger2.setLevel(logging.DEBUG)

    handler = StreamHandler(stream=sys.stdout)
    handler.setFormatter(Formatter(fmt='[%(asctime)s: %(levelname)s] %(message)s'))

    adapter = CustomLoggerAdapter(logger2, {'username': 'adilkhash'})

    logger2.addHandler(handler)
    adapter.error('failed to save')

    # extra и не только
    logger.debug('debug info', extra={"response": response.text})
    Formatter(fmt='[%(asctime)s: %(levelname)s] %(message)s, response: %(response)s')

# Конфигурация logging
Официальная документация рекомендует конфигурировать logging через python-словарь.
Для этого необходимо вызвать функцию logging.config.dictConfig и передать ей специальный словарь.
Схема словаря описана здесь. Я лишь вкратце пробегусь по основным ключам:
version - 
    ключ указывает версию конфига, рекомендуется наличие этого ключа со значением 1, нужно для обратной совместимости в случае, если в будущем появятся новые версии конфигов.
disable_existing_loggers - 
    запрещает или разрешает настройки для существующих логеров (на момент запуска), по умолчанию равен True
formatters - 
    настройки форматов логов
handlers - 
    настройки для обработчиков логов
loggers - 
    настройки существующих логеров

LOGGING_CONFIG = {
    'version': 1,
    'disable_existing_loggers': False,

    'formatters': {
        'default_formatter': {
            'format': '[%(levelname)s:%(asctime)s] %(message)s'
        },
    },

    'handlers': {
        'stream_handler': {
            'class': 'logging.StreamHandler',
            'formatter': 'default_formatter',
        },
    },

    'loggers': {
        'my_logger': {
        'handlers': ['stream_handler'],
        'level': 'DEBUG',
        'propagate': True
        }
    }
}

logging.config.dictConfig(LOGGING_CONFIG)
logger = logging.getLogger('my_logger')
logger.debug('debug log')

# Наследование в logging
    Ещё одним удобным механизмом в logging является "наследование" настроек корневого логера
    его потомками. Наследование задаётся через символ . в названии логера.
    То есть логер с названием my_package.logger1 унаследует все настройки, заданные для my_package.
    Давайте обновим пример выше, добавив в LOGGING_CONFIG настройку для my_package
    
LOGGING_CONFIG['loggers'].update (
    {
    'my_package': {
        'handlers': ['stream_handler'],
        'level': 'DEBUG',
        'propagate': False
        }
    }
)

Available format attributes:
args        You shouldn’t need to format this yourself.
    The tuple of arguments merged into msg to produce message, or a dict whose values are used for the merge (when there is only one argument, and it is a dictionary).
exc_info    You shouldn’t need to format this yourself.
    Exception tuple (à la sys.exc_info) or, if no exception has occurred, None.
msg         You shouldn’t need to format this yourself.
    The format string passed in the original logging call. Merged with args to produce message, or an arbitrary object (see Using arbitrary objects as messages).
stack_info  You shouldn’t need to format this yourself.
    Stack frame information (where available) from the bottom of the stack in the current thread, up to and including the stack frame of the logging call which resulted in the creation of this record.

%(msg)s         Message passed to logging call (same as %(message)s)
%(hostname)s    System hostname
%(username)s    System username
%(programname)s System programname

%(asctime)s     Time as human-readable string, when logging call was issued
%(created)f     Time as float when logging call was issued
%(filename)s    File name
%(funcName)s    Name of function containing the logging call
%(levelname)s   Text logging level
%(levelno)s     Integer logging level
%(lineno)d      Line number where the logging call was issued
%(message)s     Message passed to logging call (same as %(msg)s)
%(module)s      File name without extension where the logging call was issued
%(msecs)d       Millisecond part of the time when logging call was issued
%(name)s        Logger name
%(pathname)s    Full pathname to file containing the logging call
%(process)d     Process ID
%(processName)s Process name
%(relativeCreated)d - Time as integer in milliseconds when logging call was issued, relative to the time when logging module was loaded
%(thread)d      Thread ID
%(threadName)s  Thread name

%(asctime)s     Human-readable time when the LogRecord was created. By default this is of the form ‘2003-07-08 16:49:45,896’ (the numbers after the comma are millisecond portion of the time).
%(created)f     Time when the LogRecord was created (as returned by time.time()).
%(filename)s    Filename portion of pathname.
%(funcName)s    Name of function containing the logging call.
%(levelname)s   Text logging level for the message ('DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL').
%(levelno)s     Numeric logging level for the message (DEBUG, INFO, WARNING, ERROR, CRITICAL).
%(lineno)d      Source line number where the logging call was issued (if available).
%(message)s     The logged message, computed as msg % args. This is set when Formatter.format() is invoked.
%(module)s      Module (name portion of filename).
%(msecs)d       Millisecond portion of the time when the LogRecord was created.
%(name)s        Name of the logger used to log the call.
%(pathname)s    Full pathname of the source file where the logging call was issued (if available).
%(process)d     Process ID (if available).
%(processName)s Process name (if available).
%(relativeCreated)d    Time in milliseconds when the LogRecord was created, relative to the time the logging module was loaded.
%(thread)d      Thread ID (if available).
%(threadName)s  Thread name (if available).

Emoji
You can use colors for text as others mentioned in their answers to have colorful text with a background or foreground color.
But you can use emojis instead! for example, you can use ⚠️ for warning messages and 🛑 for error messages.
Or simply use these notebooks as a color:

print("📕: error message")
print("📙: warning message")
print("📗: ok status message")
print("📘: action message")
print("📓: canceled status message")
print("📔: Or anything you like and want to recognize immediately by color")

🎁 Bonus:
This method also helps you to quickly scan and find logs directly in the source code.

How to open emoji picker?
mac os: control + command + space
windows: win + .
linux: control + . or control + ;

# Отправляем логи в Telegram
    
"""
#endclass

#-------------------------------------------------
# Функциональное исполнение
#-------------------------------------------------

def GetLogDirLogon () -> str:
    """GetLogDirLogon"""
#beginfunction
    """
    if @LDomain <> ''
        LogDir = LogDir + '\\'+@LDomain
    endif
    s = AddCharR ('_', $USERID, 15)
    LogFile = s + "_" + UCase (@WKSTA)+'.log'
    """
    return ''
#endfunction

def GetLogFileName () -> str:
    """GetLogFileName"""
#beginfunction
    LResult = LUDateTime.Now ().strftime ('%Y%m%d') + '.log'
    return LResult
#endfunction

def GetLogFileNameSufix (ASufix: str) -> str:
    """GetLogFileNameSufix"""
#beginfunction
    LResult = LUDateTime.Now ().strftime ('%Y%m%d') + ASufix + '.log'
    return LResult
#endfunction

#-------------------------------------------------
# LogFileName(ALog: str, ALogDir: str, ALogFile: str) -> str:
#-------------------------------------------------
def LogFileName(ALog: int, ALogDir: str, ALogFile: str) -> str:
    """LogFileName"""
#beginfunction
    LToday: datetime = LUDateTime.Now ()
    match ALog:
        case 1|3|10|30:
            LLogDir = ALogDir
            if len (ALogDir) == 0:
                # LLogDir = os.environ['TEMP']
                LLogDir = LUFile.GetTempDir()
            #endif
            LLogFile = ALogFile
            if ALogFile == '':
                s = LUDateTime.DateTimeStr (False, LToday, LUDateTime.cFormatDateYYMMDD_01, False)
                LLogFile = s+'.log'
            #endif
            LLogFileName = os.path.join(LLogDir, LLogFile)
            if ALog == 10 or ALog == 30:
                if LUFile.FileExists(LLogFileName):
                    try:
                        LUFile.FileDelete(LLogFileName)
                    except:
                    # except LUErrors.LUFileError_FileERROR as ERROR:
                        s = f'Ошибка при удалении файла {LLogFileName}'
                        LoggerTOOLS.error (s)
                    else:
                        ...
                #endif
            #endif
        case _:
            LLogFileName = ""
    #endmatch
    return LLogFileName
#endfunction

#--------------------------------------------------------------------------------
# LogAdd (ALog: int, ALogFile: str, AOpt: str, AMessage: str,
#           AStyles = '', AFG8 = '', ABG8 = '', AFG256 = '', ABG256 = '', AESC = ''):
#--------------------------------------------------------------------------------
def LogAdd (ALog: int, ALogFile: str, AOpt: TTypeLogString, AMessage: str):
    """LogAdd"""

    def _WriteConsole(_s, T):
    #beginfunction
        if not LUSupport.IsTerminal ():
            LCOLOR = COLORS_tls.get (T)
            if LCOLOR is not None:
                LFmt = LUConsole.sBEGIN_oct + LCOLOR + _s + LUConsole.sRESET
            else:
                LFmt = _s
            LUConsole.WriteLN (LFmt)
        else:
            LCOLOR = COLORS_tls_rich.get (T)
            LFmt_default = _s
            LFmt = '[' + LCOLOR + ']' + _s
            GConsoleRich.print (LFmt)
        #endif
    #endfunction

    def _WriteFile(_s):
    #beginfunction

        LEncoding = LUFile.GetFileEncoding (ALogFile)
        if LEncoding == '':
            LEncoding = LUFile.cDefaultEncoding

        LEncoding = LUFile.cDefaultEncoding
        # sWIN = s.encode (encoding = 'UTF-8').decode(encoding = 'ANSI')
        # sWIN = s.encode (encoding = 'WINDOWS-1251').decode(encoding = 'UTF-8')
        # Это работает !!!!!!!!!!!!!!!!!
        try:
            with open (ALogFile, 'a+', encoding = LEncoding) as LFile:
                LFile.write (_s+'\n')
        except:
            _s = f'_WriteFile: Неправильная кодировка журнала!'
            LoggerTOOLS.error (_s)
        #endtry
    #endfunction

#beginfunction
    LToday = LUDateTime.Now ()
    if AOpt == TTypeLogString.tlsTEXT:
        s = AMessage
    else:
        s = LUDateTime.DateTimeStr(False, LToday, LUDateTime.cFormatDateTimeLog01, True)+' '+\
            AOpt.value+' '+AMessage
    match ALog:
        case 1|10:
            _WriteConsole (s, AOpt)
            _WriteFile (s)
        case 2:
            _WriteConsole (s, AOpt)
        case 3|30:
            _WriteConsole (s, AOpt)
            _WriteFile (s)
    #endmatch
#endfunction

#--------------------------------------------------------------------------------
# LogAddFile (ALog: int, ALogFile: str, AOpt: str, AFileName: str,
#            AStyles='', AFG8='', ABG8='', AFG256='', ABG256='', AESC=''):
#--------------------------------------------------------------------------------
def LogAddFile (ALog: int, ALogFile: str, AOpt: TTypeLogString, AFileName: str):
    """LogAddFile"""
#beginfunction
    if LUFile.FileExists (AFileName):
        try:
            LEncoding = LUFile.GetFileEncoding (AFileName)
            if LEncoding == '':
                LEncoding = LUFile.cDefaultEncoding
            with open (AFileName, 'r', encoding = LEncoding) as LFile:
                for s in LFile:
                    Ls = s.split ('\n')[0]
                    LogAdd (ALog, ALogFile, AOpt, Ls)
                #endfor
            #endwith
        except:
            s = f'LogAddFile: Неправильная кодировка журнала!'+AFileName
            LoggerTOOLS.error (s)
        #endtry
    #endif
#endfunction

#-------------------------------------------------
# GLOBAL
#-------------------------------------------------
def SetFormatterForLogger (ALogger: logging.Logger):
    """SetFormatterForLogger"""
#beginfunction
    for item in ALogger.handlers:
        if type (item.formatter) is pythonjsonlogger.jsonlogger.JsonFormatter:
            item.formatter.json_ensure_ascii = False
        #endif
        if type (item) is logging.StreamHandler or type (item) is TStreamHandler:
            # if LUSupport.ISTerminal ():
            #     TStreamHandler(item).setLevel(logging.CRITICAL)
            #     # print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
            #     ...
            # #endif
            Lfmt = item.formatter._fmt
            Ldatefmt = item.formatter.datefmt
            LFormaterConsole = TFormatter (fmt = Lfmt, datefmt = Ldatefmt)
            item.setFormatter (LFormaterConsole)
        #endif
    #enfor
#endfunction

def PrintHandlers (ALogger: logging.Logger):
    """Printhandlers"""
#beginfunction
    for item in ALogger.root.handlers:
        s = f'{item.name}={item}'
        # LoggerTOOLS.info(s)
        if type(item) is logging.StreamHandler:
            LoggerTOOLS.info ('logging.StreamHandler='+s)
            # Lfmt = item.formatter._fmt
            # Ldatefmt = item.formatter.datefmt
            # LFormaterConsole = TFormatter (AUseColor=True, fmt=Lfmt, datefmt=Ldatefmt)
            ...
        #endif
        if type (item) is TStreamHandler:
            LoggerTOOLS.info ('TStreamHandler='+s)
            # Lfmt = item.formatter._fmt
            # Ldatefmt = item.formatter.datefmt
            # LFormaterConsole = TFormatter (AUseColor=True, fmt=Lfmt, datefmt=Ldatefmt)
            ...
        #endif
    #enfor
#endfunction

def GetHandler (ALogger: logging.Logger, ANameHandler: str):
    """Printhandlers"""
#beginfunction
    for item in ALogger.root.handlers:
        # s = f'{item.name}={item}'
        # LoggerTOOLS.info(s)
        if item.name == ANameHandler:
            return item
        #endif
    #enfor
#endfunction

def WinToUnix (Astr: str) -> str:
    """WinToUnix"""
#beginfunction
    LOSInfo = LUos.TOSInfo ()
    print ('HostName      = ' + LOSInfo.node)
    print ('OS            = ' + LOSInfo.system)
    # print ('OS            = ' + LOSInfo.uname.system)
    match LOSInfo.system.upper ():
        case 'LINUX':
            Lstr = os.path.abspath (Astr).replace ("\\", "/")
        case 'WINDOWS':
            Lstr = Astr
        case _:
            # print ('INFO: Only LINUX or WINDOWS')
            Lstr = Astr
    #endmatch
    return Lstr
#endfunction

#-------------------------------------------------
# LOGGING_CONFIG
#-------------------------------------------------
LOGGING_CONFIG = \
{
    'version': 1,
    'disable_existing_loggers': 1,
    'loggers': {
        'root': {
            'handlers': [
                'CONSOLE',
                'FILE_01'
            ],
            'level': 'DEBUG',
            'propagate': 1
        },
        'log01': {
            'handlers': [
                'FILE_01'
            ],
            'level': 'DEBUG',
            'propagate': 0,
            'qualname': 'log01'
        },
        'log02': {
            'handlers': [
                'FILE_02'
            ],
            'level': 'DEBUG',
            'propagate': 0,
            'qualname': 'log02'
        }
    },
    'handlers': {
        'CONSOLE': {
            # 'class': 'logging.StreamHandler',
            'class': 'LULog.TStreamHandler',
            'level': 'DEBUG',
            'formatter': 'FORMAT_01',
            'stream': 'ext://sys.stdout'
        },
        'FILE_01': {
            'class': 'logging.handlers.RotatingFileHandler',
            'level': 'DEBUG',
            'formatter': 'FORMAT_01',
            'maxBytes': 10000000,
            'backupCount': 5,
            'filename': 'LOGGING_CONFIG.log'
        },
        'FILE_02': {
            # 'class': 'logging.handlers.TimedRotatingFileHandler',
            'class': 'logging.handlers.RotatingFileHandler',
            'level': 'INFO',
            'formatter': 'FORMAT_json',
            # 'interval': 'M',
            'maxBytes': 1024,
            'backupCount': 5,
            'filename': 'LOGGING_CONFIG_json.log'
        }
    },
    'formatters': {
        'FORMAT_01': {
            'format': Cstrfmt_01,
            'datefmt': Cdatefmt_01
        },
        'FORMAT_json': {
            'class': 'pythonjsonlogger.jsonlogger.JsonFormatter',
            'format': Cstrfmt_01,
            'datefmt': Cdatefmt_01
        }
    },
}

#-------------------------------------------------
# CreateLoggerCONFIG
#-------------------------------------------------
def CreateLoggerCONFIG (AFileNameCONFIG: str, ALogerName: str,
                        ADirectoryLOG: str, AFileNameLOG: str, AFileNameLOGjson: str) -> logging.Logger:
    """CreateLoggerCONFIG"""
#beginfunction
    global CONFIG
    CONFIG = {}
    global LoggerTOOLS

    #print ('CONFIG:ADirectoryLOG:',ADirectoryLOG)

    LPath = LUFile.ExtractFileDir(__file__)
    LFileNameCONFIG = os.path.join (LPath, AFileNameCONFIG)
    if LUFile.FileExists(LFileNameCONFIG):
        # читаем конфигурацию из файла
        try:
            with open (LFileNameCONFIG, 'r') as FileCONFIG:
                CONFIG = json.load(FileCONFIG)
            #endwith
        except FileNotFoundError as ERROR:
            print ('ERROR: Невозможно открыть файл', ERROR)
        #endtry
    else:
        CONFIG = copy.deepcopy (LOGGING_CONFIG)
    #endif
    #-------------------------------------------------------------------
    # CONFIG = copy.deepcopy (LOGGING_CONFIG)
    #-------------------------------------------------------------------

    if AFileNameLOG == '':
        LOptionValue_01 = CONFIG['handlers']['FILE_01']['filename']
        # print ('LOptionValue_01:',LOptionValue_01)
        LFileNameLOG = LUFile.ExtractFileName (LOptionValue_01)
    else:
        LFileNameLOG = LUFile.ExtractFileName (AFileNameLOG)
    #endif
    # print('LFileNameLOG:',LFileNameLOG)

    if AFileNameLOGjson == '':
        LOptionValue_02 = CONFIG['handlers']['FILE_02']['filename']
        # print ('LOptionValue_02:',LOptionValue_02)
        LFileNameLOGjson = LUFile.ExtractFileName (LOptionValue_02)
    else:
        LFileNameLOGjson = LUFile.ExtractFileName (AFileNameLOGjson)
    #endif
    # print('LFileNameLOGjson:',LFileNameLOGjson)

    if ADirectoryLOG == '':
        # log будет создан в текущем каталоге (по умолчанию)
        LDirectoryLOG = LUos.GetCurrentDir ()
    else:
        # log будет создан в ADirectoryLOG
        LDirectoryLOG = LUFile.ExpandFileName (ADirectoryLOG)
    #endif
    # print('LDirectoryLOG:',LDirectoryLOG)
    if not LUFile.DirectoryExists (LDirectoryLOG):
        LUFile.ForceDirectories(LDirectoryLOG)
    #endif

    # установить имена log файлов в CONFIG
    LOptionValue_01 = os.path.join (LDirectoryLOG, LFileNameLOG)
    # print('LOptionValue_01:', LOptionValue_01)
    CONFIG ['handlers'] ['FILE_01'] ['filename'] = LOptionValue_01

    LOptionValue_02 = os.path.join (LDirectoryLOG, LFileNameLOGjson)
    # print('LOptionValue_02:', LOptionValue_02)
    CONFIG ['handlers'] ['FILE_02'] ['filename'] = LOptionValue_02

    if len(CONFIG) > 0:
        #-------------------------------------------------------------------
        # LFileNameCONFIG = os.path.join (LUos.GetCurrentDir (), CDefaultFileLogCONFIG)
        LFileNameCONFIG = os.path.join (LUFile.GetTempDir(), CDefaultFileLogCONFIG)

        LUDict.SaveDictSTR (CONFIG, LFileNameCONFIG)
        #-------------------------------------------------------------------
        # читаем конфигурацию из словаря
        logging.config.dictConfig (CONFIG)
        # создаем регистратор
        LResult = logging.getLogger (ALogerName)
        # установить форматер
        SetFormatterForLogger (LResult)
        return LResult
    else:
        return None
#endfunction

#-------------------------------------------------
# CreateLoggerYAML
#-------------------------------------------------
def CreateLoggerYAML (AFileNameYAML: str, ALogerName: str, ADirectoryLOG: str, AFileNameLOG: str, AFileNameLOGjson: str) -> logging.Logger:
    """CreateLoggerFILEYAML"""
#beginfunction
    global CONFIG_YAML
    CONFIG_YAML = {}

    LPath = LUFile.ExtractFileDir(__file__)
    LFileNameYAML = os.path.join (LPath, AFileNameYAML)
    if LUFile.FileExists (LFileNameYAML):
        # читаем конфигурацию из файла
        try:
            with (open (LFileNameYAML, 'r') as FileCONFIG_YAML):
                CONFIG_YAML = yaml.load(FileCONFIG_YAML, Loader=yaml.FullLoader)
            #endwith
        except FileNotFoundError as ERROR:
            print ('ERROR: Невозможно открыть файл', ERROR)
        #endtry
    else:
        CONFIG_YAML = copy.deepcopy (LOGGING_CONFIG)
    #endif
    #-------------------------------------------------------------------
    # CONFIG_YAML = copy.deepcopy (LOGGING_CONFIG)
    #-------------------------------------------------------------------

    if AFileNameLOG == '':
        LOptionValue_01 = CONFIG_YAML ['handlers'] ['FILE_01'] ['filename']
        # print ('LOptionValue_01:', LOptionValue_01)
        LFileNameLOG = LUFile.ExtractFileName (LOptionValue_01)
    else:
        LFileNameLOG = LUFile.ExtractFileName (AFileNameLOG)
    #endif
    # print ('LFileNameLOG:', LFileNameLOG)

    if AFileNameLOGjson == '':
        LOptionValue_02 = CONFIG_YAML ['handlers'] ['FILE_02'] ['filename']
        # print ('LOptionValue_02:', LOptionValue_02)
        LFileNameLOGjson = LUFile.ExtractFileName (LOptionValue_02)
    else:
        LFileNameLOGjson = LUFile.ExtractFileName (AFileNameLOGjson)
    #endif
    # print ('LFileNameLOGjson:', LFileNameLOGjson)

    if ADirectoryLOG == '':
        # log будет создан в текущем каталоге (по умолчанию)
        LDirectoryLOG = LUos.GetCurrentDir ()
    else:
        # log будет создан в ADirectoryLOG
        LDirectoryLOG = LUFile.ExpandFileName (ADirectoryLOG)
    #endif
    # print('LDirectoryLOG:',LDirectoryLOG)
    if not LUFile.DirectoryExists (LDirectoryLOG):
        LUFile.ForceDirectories(LDirectoryLOG)
    #endif

    # установить имена log файлов в CONFIG
    LOptionValue_01 = os.path.join (LDirectoryLOG, LFileNameLOG)
    # print('LOptionValue_01:', LOptionValue_01)
    CONFIG_YAML ['handlers'] ['FILE_01'] ['filename'] = LOptionValue_01
    LOptionValue_02 = os.path.join (LDirectoryLOG, LFileNameLOGjson)
    # print('LOptionValue_02:', LOptionValue_02)
    CONFIG_YAML ['handlers'] ['FILE_02'] ['filename'] = LOptionValue_02

    if len (CONFIG_YAML) > 0:
        #-------------------------------------------------------------------
        LFileNameYAML = os.path.join (LUos.GetCurrentDir (), CDefaultFileLogYAML)
        LUDict.SaveDictSTR (CONFIG_YAML, LFileNameYAML)
        #-------------------------------------------------------------------
        # читаем конфигурацию из словаря
        logging.config.dictConfig (CONFIG_YAML)
        # создаем регистратор
        LResult = logging.getLogger (ALogerName)
        # установить форматер
        SetFormatterForLogger (LResult)
        return LResult
    else:
        return None
    #endif

#endfunction

#-------------------------------------------------
# CreateLoggerFILEINI
#-------------------------------------------------
def CreateLoggerFILEINI (AFileNameINI: str, ALogerName: str,
                         ADirectoryLOG: str, AFileNameLOG: str, AFileNameLOGjson: str) -> logging.Logger:
    """CreateLoggerFILEINI"""
#beginfunction
    LDirectoryLOG = ADirectoryLOG

    # читаем конфигурацию из файла INI
    LFileNameINI = LUFile.ExpandFileName (AFileNameINI)

    if LUFile.FileExists (LFileNameINI):
        # существует файл, который можно редактировать
        SetEditINI = True
        LPathINI = LUFile.ExtractFileDir (LFileNameINI)
        LFileNameINI = os.path.join (LPathINI, LUFile.ExtractFileName (AFileNameINI))
    else:
        SetEditINI = False
        if not ALogerName == 'console':
            LPathINI = LUos.GetCurrentDir ()
            LFileNameINI = os.path.join (LPathINI, LUFile.ExtractFileName (AFileNameINI))
            if LUFile.FileExists (LFileNameINI):
                # существует файл в текущем каталоге, который можно редактировать
                SetEditINI = True
            else:
                # берем имя файла из проекта, если оно есть
                SetEditINI = True
                LPathINI = LUFile.ExtractFileDir (__file__)
                LFileNameINIorig = os.path.join (LPathINI, CDefaultFileLogINI)
                # Копирование файла
                shutil.copy (LFileNameINIorig, LFileNameINI)
            #endif
        else:
            LPathINI = LUFile.ExtractFileDir (__file__)
            LFileNameINI = os.path.join (LPathINI, LUFile.ExtractFileName (AFileNameINI))
        #endif
    #endif

    # print ('LFileNameINI:', LFileNameINI)

    if not SetEditINI:
        # print ('ALogerName:',ALogerName)
        pass
    else:
        # print ('AFileNameLOG:', AFileNameLOG)
        LINIFile = LUParserINI.TINIFile ()
        LINIFile.FileNameINI = LFileNameINI
        LOptionName = 'args'
        if AFileNameLOG == '':
            LSectionName_01 = 'handler_FILE_01'
            LOptionValue_01 = LINIFile.GetOption(LSectionName_01, LOptionName, '')
            # print ('LOptionValue_01:',LOptionValue_01)
            LFileNameLOG = LOptionValue_01.split("'")[1]
            # print('LFileNameLOG:',LFileNameLOG)
            LFileNameLOG = LUFile.ExtractFileName (LFileNameLOG)
        else:
            LFileNameLOG = LUFile.ExtractFileName (AFileNameLOG)
        #endif
        # print('LFileNameLOG:',LFileNameLOG)

        if AFileNameLOGjson == '':
            LSectionName_02 = 'handler_FILE_02'
            LOptionValue_02 = LINIFile.GetOption(LSectionName_02, LOptionName, '')
            # print ('LOptionValue_02:',LOptionValue_02)
            LFileNameLOGjson = LOptionValue_02.split("'")[1]
            # print('LFileNameLOGjson:',LFileNameLOGjson)
            LFileNameLOGjson = LUFile.ExtractFileName (LFileNameLOGjson)
        else:
            LFileNameLOGjson = LUFile.ExtractFileName (AFileNameLOGjson)
        #endif
        print('AFileNameLOGjson:',AFileNameLOGjson)
        print('LFileNameLOGjson:',LFileNameLOGjson)

        # установить имена log файлов в ini
        LOptionValue_01 = "('" + os.path.join (LDirectoryLOG, LFileNameLOG) + "',)"
        if LUos.GOSInfo.system == 'Windows':
            LOptionValue_01 = LOptionValue_01.replace ('\\', "\\\\")
        #endif
        if LUos.GOSInfo.system == 'Linux':
            raise 'ERROR: Linux не поддерживается'
        #endif
        # print('LOptionValue_01:',LOptionValue_01)
        LINIFile.SetOption ('handler_FILE_01', LOptionName, LOptionValue_01)
        LOptionValue_02 = "('" + os.path.join (LDirectoryLOG, LFileNameLOGjson) + "',)"
        if LUos.GOSInfo.system == 'Windows':
            LOptionValue_02 = LOptionValue_02.replace ("\\", "\\\\")
        #endif
        if LUos.GOSInfo.system == 'Linux':
            raise 'ERROR: Linux не поддерживается'
        #endif
        # print('LOptionValue_02:',LOptionValue_02)

        LINIFile.SetOption ('handler_FILE_02', LOptionName, LOptionValue_02)
        LINIFile.UpdateFileINI ()
    #endif

    # print ('INI:ADirectoryLOG:',ADirectoryLOG)
    if ADirectoryLOG == '':
        # log будет создан в текущем каталоге (по умолчанию)
        LDirectoryLOG = LUos.GetCurrentDir ()
    else:
        # log будет создан в ADirectoryLOG
        LDirectoryLOG = LUFile.ExpandFileName (ADirectoryLOG)
    #endif
    # print('...LDirectoryLOG:',LDirectoryLOG)

    if not LUFile.DirectoryExists (LDirectoryLOG):
        LUFile.ForceDirectories(LDirectoryLOG)
    #endif

    # os.chdir (LDirectoryLOG)

    # print(LFileNameINI)

    logging.config.fileConfig (LFileNameINI, disable_existing_loggers=True,
                               encoding=LUFile.cDefaultEncoding)
    # logging.config.fileConfig (LFileNameINI, disable_existing_loggers=True, encoding='cp1251')

    # создаем регистратор
    LResult = logging.getLogger (ALogerName)
    # print(f'{LResult.name=}')
    # print(f'{LResult.handlers=}')
    # установить форматер
    SetFormatterForLogger (LResult)

    return LResult
#endfunction

#-------------------------------------------------
# CreateLoggerBASIC
#-------------------------------------------------
def CreateLoggerBASIC (ALevel, AFileNameLOG: str, ALogerName: str) -> logging.Logger:
    """CreateTLoggingCONFIG"""
#beginfunction
    # читаем конфигурацию из
    if len(AFileNameLOG) > 0:
        logging.basicConfig (level = ALevel, filename = AFileNameLOG, style='%',
                             datefmt = Cdatefmt_01, format = Cstrfmt_01)
    else:
        logging.basicConfig (level = ALevel, stream=sys.stdout, style='%',
                             datefmt = Cdatefmt_01, format = Cstrfmt_01)
    # создаем регистратор
    LResult = logging.getLogger (ALogerName)
    # установить форматер
    SetFormatterForLogger (LResult)
    return LResult
#endfunction

#-------------------------------------------------
# LoggerTLogger
#-------------------------------------------------
def CreateTLogger (ALogerName: str) -> TLogger:
    """CreateTLogging"""
#beginfunction
    logging.setLoggerClass (TLogger)
    # создаем регистратор
    LResult = TLogger(ALogerName)
    SetFormatterForLogger (LResult)
    return LResult
#endfunction

#-------------------------------------------------
# FileMemoLog
#-------------------------------------------------
def CreateTFileMemoLog () -> TFileMemoLog:
    """CreateTFileMemoLog"""
#beginfunction
    LFileMemoLog = TFileMemoLog ()
    return LFileMemoLog
#endfunction

#-------------------------------------------------
# Инициализация системы logging
#-------------------------------------------------
STATLogging = True

GLoggerFILEINI = logging.Logger
GLoggerCONFIG = logging.Logger
LoggerTOOLS = logging.Logger
LoggerAPPS = logging.Logger
LoggerTLogger = TLogger
FileMemoLog = TFileMemoLog

def STARTLogging (T: TTypeSETUPLOG, ALogerName, ADirectoryLOG: str, AFileNameLOG: str, AFileNameLOGjson: str) -> None:
    """STARTLogging"""
#beginfunction
    global GLoggerFILEINI
    global GLoggerCONFIG
    global LoggerTOOLS
    global LoggerAPPS
    global LoggerTLogger
    global FileMemoLog
    global STATLogging

    STATLogging = False

    # print (sys._getframe (0).f_code.co_name, '...')
    # print (inspect.currentframe().f_code.co_name, '...')
    # print (inspect.stack () [0] [3], '...')
    # print (traceback.extract_stack () [-1].name, '...')

    LLogerNames = ['root','log01','log01', 'console']

    AddLevelName ()

    LT = T
    if LUos.GOSInfo.system == 'Windows':
        LT = T
    #endif
    if LUos.GOSInfo.system == 'Linux':
        LT = TTypeSETUPLOG.tslCONFIG
    #endif

    if ALogerName in LLogerNames:
        match LT:
            case TTypeSETUPLOG.tslCONFIG:
                GLoggerCONFIG = CreateLoggerCONFIG (CDefaultFileLogCONFIG, ALogerName,
                                                    ADirectoryLOG, AFileNameLOG,
                                                    AFileNameLOGjson)
            case TTypeSETUPLOG.tslYAML:
                GLoggerYAML = CreateLoggerYAML (CDefaultFileLogYAML, ALogerName,
                                                ADirectoryLOG, AFileNameLOG,
                                                AFileNameLOGjson)
            case TTypeSETUPLOG.tslINI:
                if ALogerName == 'console' or LUConst.GAPPName is None:
                    # 'logging_CONSOLE.INI'
                    LFileLogINI = CDefaultFileLogINI
                    LFileLogINI = CDefaultFileLogINI_CONSOLE
                else:
                    LFileLogINI = LUConst.GAPPName+'.ini'
                #endif
                GLoggerFILEINI = CreateLoggerFILEINI (LFileLogINI, ALogerName,
                                                      ADirectoryLOG, AFileNameLOG,
                                                      AFileNameLOGjson)
            case _:
                pass
        #endmatch
    else:
        exit()
    #endif

    #-------------------------------------------------
    # GLoggerBASIC = CreateLoggerBASIC (logging.DEBUG, 'LOG\\' + CDefaultFileLogFILEBASIC, 'root')
    # GLoggerBASIC = CreateLoggerBASIC (logging.DEBUG, '', 'root')
    #-------------------------------------------------

    #-------------------------------------------------
    # LoggerTOOLS
    #-------------------------------------------------
    CLoggerTOOLS = 'TOOLS__'
    LoggerTOOLS = logging.getLogger (CLoggerTOOLS)
    # LoggerTOOLS.disabled = False
    # LoggerTOOLS.info ('info')
    # print('LoggerTOOLS' in vars () or 'LoggerTOOLS' in globals ())
    # print('LoggerTOOLS' in vars ())
    # print('LoggerTOOLS' in globals ())
    # if not ('LoggerTOOLS' in vars () or 'LoggerTOOLS' in globals ()):
    #     CLoggerTOOLS = 'TOOLS__'
    #     LoggerTOOLS = logging.getLogger (CLoggerTOOLS)
    # #endif

    #-------------------------------------------------
    # LoggerAPPS
    #-------------------------------------------------
    CLoggerAPPS = 'APPS__'
    LoggerAPPS = logging.getLogger(CLoggerAPPS)

    #-------------------------------------------------
    # LoggerTLogger
    #-------------------------------------------------
    CTLogger = 'TLOGGER'
    LoggerTLogger = CreateTLogger (CTLogger)

    #-------------------------------------------------
    # FileMemoLog
    #-------------------------------------------------
    FileMemoLog = CreateTFileMemoLog ()

    #-------------------------------------------------
    # Отключить журнал 'chardet.charsetprober'
    #-------------------------------------------------
    logger = logging.getLogger('chardet.charsetprober')
    logger.setLevel(logging.INFO)
    logger = logging.getLogger('chardet.universaldetector')
    logger.setLevel(logging.INFO)

    #-------------------------------------------------
    # Отключить журнал 'pytube.extract'
    #-------------------------------------------------
    logger = logging.getLogger('pytube.extract')
    logger.setLevel(logging.INFO)
    
    #-------------------------------------------------
    # Отключить журнал 'pytube.streams'
    #-------------------------------------------------
    logger = logging.getLogger('pytube.streams')
    logger.setLevel(logging.INFO)
    
    #-------------------------------------------------
    # Отключить журнал 'pytube.cipher'
    #-------------------------------------------------
    logger = logging.getLogger('pytube.cipher')
    logger.setLevel(logging.INFO)
    
    #-------------------------------------------------
    # Отключить журнал 'pytube.helpers'
    #-------------------------------------------------
    logger = logging.getLogger('pytube.helpers')
    logger.setLevel(logging.INFO)

    STATLogging = True
#endfunction

#-------------------------------------------------
# Выключить систему logging
#-------------------------------------------------
def STOPLogging () -> None:
    """STOPLogging"""
#beginfunction
    global STATLogging
    # global LoggerTOOLS
    STATLogging = False
    # LoggerTOOLS.disabled = True# Выключить систему logging для логгирования
#endfunction

#-------------------------------------------------
# LoggerAdd
#-------------------------------------------------
def LoggerAdd (ALogger, ALevel, Astr):
#beginfunction
    if STATLogging:
        try:
            ALogger.log(ALevel, Astr)
        except:
            print("ERROR:")
            ...
        #endtry
    else:
        print("INFO: система не включена для записи логов")
    #endif
#endfunction

# #-------------------------------------------------
# # LoggerTOOLS_AddLevel
# #-------------------------------------------------
# def LoggerTOOLS_AddLevel (ALevel, Astr):
# #beginfunction
#     if STATLogging:
#         try:
#             LoggerTOOLS.log(ALevel, Astr)
#         except:
#             ...
#         #endtry
#     else:
#         print("INFO: система не включена для записи логов")
#     #endif
# #endfunction
#
# #-------------------------------------------------
# # LoggerTOOLS_AddDebug
# #-------------------------------------------------
# def LoggerTOOLS_AddDebug (Astr):
# #beginfunction
#     if STATLogging:
#         try:
#             LoggerTOOLS.debug(Astr)
#         except:
#             ...
#         #endtry
#     else:
#         print("INFO: система не включена для записи логов")
#     #endif
# #endfunction
#
# #-------------------------------------------------
# # LoggerTOOLS_AddError
# #-------------------------------------------------
# def LoggerTOOLS_AddError (Astr):
# #beginfunction
#     if STATLogging:
#         try:
#             LoggerTOOLS.error(Astr)
#         except:
#             ...
#         #endtry
#     else:
#         print("INFO: система не включена для записи логов")
#     #endif
# #endfunction
#
# #-------------------------------------------------
# # LoggerAPPS_AddLevel
# #-------------------------------------------------
# #LULog.LoggerAPPS.log
# def LoggerAPPS_AddLevel (ALevel, Astr):
# #beginfunction
#     if STATLogging:
#         try:
#             LoggerAPPS.log(ALevel, Astr)
#         except:
#             ...
#         #endtry
#     else:
#         print("INFO: система не включена для записи логов")
#     #endif
# #endfunction
#
# #-------------------------------------------------
# # LoggerAPPS_AddInfo
# #-------------------------------------------------
# #LULog.LoggerAPPS.info
# def LoggerAPPS_AddInfo (Astr):
# #beginfunction
#     if STATLogging:
#         try:
#             LoggerAPPS.info(Astr)
#         except:
#             ...
#         #endtry
#     else:
#         print("INFO: система не включена для записи логов")
#     #endif
# #endfunction
#
# #-------------------------------------------------
# # LoggerAPPS_AddError
# #-------------------------------------------------
# #LULog.LoggerAPPS.error
# def LoggerAPPS_AddError (Astr):
# #beginfunction
#     if STATLogging:
#         try:
#             LoggerAPPS.error(Astr)
#         except:
#             ...
#         #endtry
#     else:
#         print("INFO: система не включена для записи логов")
#     #endif
# #endfunction
#
# #-------------------------------------------------
# # LoggerAPPS_AddDebug
# #-------------------------------------------------
# #LULog.LoggerAPPS.debug
# def LoggerAPPS_AddDebug (Astr):
# #beginfunction
#     if STATLogging:
#         try:
#             LoggerAPPS.debug(Astr)
#         except:
#             ...
#         #endtry
#     else:
#         print("INFO: система не включена для записи логов")
#     #endif
# #endfunction

#-------------------------------------------------
# main
#-------------------------------------------------
def main ():
#beginfunction
    print('main LULog.py...')
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule

if __name__ == "__main__":
    main()
else:
    STOPLogging ()
#endif

#endmodule



"""LUParserINI.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUParserINI.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import configparser

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКА LU
#------------------------------------------
import lyrpy.LULog as LULog
import lyrpy.LUConst as LUConst
import lyrpy.LUFile as LUFile
import lyrpy.LUos as LUos
import lyrpy.LUStrDecode as LUStrDecode
import lyrpy.LUStrUtils as LUStrUtils

class TINIFile (configparser.ConfigParser):
    """TINIFile"""
    luClassName = 'TINIFile'

    #--------------------------------------------------
    # constructor
    #--------------------------------------------------
    """
    class configparser.ConfigParser (
        defaults=None,
        dict_type=dict,
        allow_no_value=False,
        delimiters=('=', ':'),
        comment_prefixes=('#', ';'),
        inline_comment_prefixes=None,
        strict=True,
        empty_lines_in_values=True,
        default_section=configparser.DEFAULTSECT,
        interpolation=BasicInterpolation(),
        converters={}
        )
    """
    @staticmethod
    def __GetINIFileName (AFileName: str) -> str:
        """__GetINIFileName"""
    #beginfunction
        LResult = ''
        P = LUFile.ExtractFileDir (AFileName)
        F = LUFile.ExtractFileName (AFileName)
        E = LUFile.ExtractFileExt (AFileName)
        if E == '':
            F = F + '.ini'
        #endif
        if P == '':
            LWinDir = LUos.GetEnvVar (LUos.cWINDIR)
            LPath = [LUos.GetCurrentDir (), LWinDir]

            LList = LUFile.SearchFileDirs (LPath, F, '', '', False)
            if len(LList) > 0:
                LResult = LList[0]
            #endif
            if LResult == '':
                LResult = LUFile.IncludeTrailingBackslash (LWinDir) + F
            #endif
        else:
            LResult = LUFile.ExpandFileName (AFileName)
        #endif
        return LResult
    #endfunction

    def __init__ (self, empty_lines_in_values=True, **kwargs):      # allow_no_value=True
        """Constructor"""
    #beginfunction
        super ().__init__ (empty_lines_in_values=True, **kwargs)
        self.__FSectionName: str = ''
        self.__FOptionName: str = ''
        self.__FOptionValue: str = ''
        self.__FChangedFileINI: bool = False
        self.__FFileNameINI: str = ''
    #endfunction

    #--------------------------------------------------
    # destructor
    #--------------------------------------------------
    def __del__ (self):
        """destructor"""
    #beginfunction
        LClassName = self.__class__.__name__
        # s = '{} уничтожен'.format (LClassName)
        # LULog.LoggerTOOLS_AddLevel (LULog.DEBUGTEXT, s)
        #print (s)
    #endfunction

    #--------------------------------------------------
    # @property ConfigParser
    #--------------------------------------------------
    # getter
    @property
    def ConfigParser(self) -> configparser.ConfigParser:
    #beginfunction
        return self
    #endfunction

    #--------------------------------------------------
    # @property Sections
    #--------------------------------------------------
    # getter
    @property
    def Sections(self):
    #beginfunction
        return self.sections()
    #endfunction

    #--------------------------------------------------
    # @property Options
    #--------------------------------------------------
    # getter
    @property
    def Options(self):
    #beginfunction
        return self.options(self.SectionName)
    #endfunction

    #--------------------------------------------------
    # @property SectionName
    #--------------------------------------------------
    # getter
    @property
    def SectionName(self):
    #beginfunction
        return self.__FSectionName
    #endfunction
    # setter
    @SectionName.setter
    def SectionName (self, AValue: str):
    #beginfunction
        self.__FSectionName = AValue
    #endfunction

    #--------------------------------------------------
    # @property OptionName
    #--------------------------------------------------
    # getter
    @property
    def OptionName(self):
    #beginfunction
        return self.__FOptionName
    #endfunction
    # setter
    @OptionName.setter
    def OptionName (self, AValue: str):
    #beginfunction
        self.__FOptionName = AValue
    #endfunction

    #--------------------------------------------------
    # @property OptionValue
    #--------------------------------------------------
    # getter
    @property
    def OptionValue(self):
    #beginfunction
        self.__FOptionValue = self.get(self.SectionName, self.OptionName)
        return self.__FOptionValue
    #endfunction
    # setter
    @OptionValue.setter
    def OptionValue (self, AValue: str):
    #beginfunction
        self.__FOptionValue = AValue
        self.set(self.SectionName, self.OptionName, AValue)
    #endfunction

    #--------------------------------------------------
    # @property FileNameINI
    #--------------------------------------------------
    # getter
    @property
    def FileNameINI(self):
    #beginfunction
        return self.__FFileNameINI
    #endfunction
    # setter
    @FileNameINI.setter
    def FileNameINI (self, AValue: str):
    #beginfunction
        LFullFileName = self.__GetINIFileName (AValue)
        if not LUFile.FileExists (LFullFileName):
            LFullFileName = LUFile.ExpandFileName (LUFile.ExtractFileName(AValue))
            LUFile.CreateTextFile (LFullFileName, '', LUStrDecode.cCP1251)
        self.__FFileNameINI = LFullFileName
        self.__OpenFileINI ()
        self.ChangedFileINI = False
    #endfunction

    #--------------------------------------------------
    # @property ChangedFileINI
    #--------------------------------------------------
    # getter
    @property
    def ChangedFileINI(self):
    #beginfunction
        return self.__FChangedFileINI
    #endfunction
    # setter
    @ChangedFileINI.setter
    def ChangedFileINI (self, AValue: bool):
    #beginfunction
        self.__FChangedFileINI = AValue
    #endfunction

    def __OpenFileINI (self):
        """__OpenFileINI"""
    #beginfunction
        self.read (self.FileNameINI)
    #endfunction

    def IsSection (self, ASectionName: str) -> bool:
        """IsSection"""
    #beginfunction
        return self.has_section(ASectionName)
    #endfunction

    def IsOption (self, ASectionName: str, AOption: str) -> bool:
        """IsOption"""
    #beginfunction
        return self.has_option(ASectionName, AOption)
    #endfunction

    def UpdateFileINI (self):
        """UpdateFileINI"""
    #beginfunction
        if self.ChangedFileINI:
            with open (self.FileNameINI, 'w', encoding = LUStrDecode.cCP1251) as LFileINI:
                self.write (LFileINI)
            #endwith
            self.__OpenFileINI ()
        #endif
    #endfunction

    def RefreashOption (self):
        """RefreashOption"""
    #beginfunction
        self.__OpenFileINI ()
    #endfunction

    def GetOption (self, ASectionName: str, AOptionName: str, AValueDefault):
        """GetOption"""
    #beginfunction
        if self.has_section(ASectionName):
            if type(AValueDefault) == int:
                try:
                    i = self.getint (ASectionName, AOptionName)
                except:
                    i = AValueDefault
                #endtry
                return i
            elif type(AValueDefault) == bool:
                s = self.get(ASectionName, AOptionName)
                return LUStrUtils.strtobool(s)
            elif type(AValueDefault) == float:
                try:
                    f = self.getfloat (ASectionName, AOptionName)
                except:
                    f = AValueDefault
                #endtry
                return f
            else:
                s = self.get(ASectionName, AOptionName)
                return s
            #endif
        else:
            return AValueDefault
        #endif
    #endfunction

    def SetOption (self, ASectionName: str, AOptionName: str, AValue):
        """SetOption"""
    #beginfunction
        if not self.has_section(ASectionName):
            self.add_section (ASectionName)
        #endif
        s = ''
        if type(AValue) == int:
            try:
                s = str (AValue)
                self.ChangedFileINI = True
            except:
                self.ChangedFileINI = False
            #endtry
        elif type(AValue) == bool:
            s = LUStrUtils.booltostr (AValue)
            self.ChangedFileINI = True
        elif type(AValue) == float:
            try:
                s = str (AValue)
                self.ChangedFileINI = True
            except:
                self.ChangedFileINI = False
            #endtry
        else:
            s = AValue
            self.ChangedFileINI = True
        #endif
        if self.ChangedFileINI:
            self.set (ASectionName, AOptionName, s)
            self.UpdateFileINI ()
        #endif
    #endfunction

    def DeleteSection (self, ASectionName: str):
        """DeleteSection"""
    #beginfunction
        if self.IsSection (ASectionName):
            self.remove_section(ASectionName)
            self.UpdateFileINI ()
        #endif
        self.ChangedFileINI = True
    #endfunction

    def DeleteOption (self, ASectionName: str, AOptionName: str):
        """DeleteOption"""
    #beginfunction
        if self.IsOption (ASectionName, AOptionName):
            self.remove_option(ASectionName, AOptionName)
            self.UpdateFileINI ()
        #endif
        self.ChangedFileINI = True
    #endfunction

#endclass

#---------------------------------------------------------
# CreateTINIFile
#---------------------------------------------------------
def CreateTINIFile () -> TINIFile:
    """CreateTINIFile"""
#beginfunction
    return TINIFile ()
#endfunction

GINIFile = CreateTINIFile ()

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#------------------------------------------
#
#------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule



"""LUSupport.py"""
# -*- coding: UTF-8 -*-
__annotations__ = """
 =======================================================
 Copyright (c) 2023-2024
 Author:
     Lisitsin Y.R.
 Project:
     LU_PY
     Python (LU)
 Module:
     LUSupport.py

 =======================================================
"""

#------------------------------------------
# БИБЛИОТЕКИ python
#------------------------------------------
import sys

#------------------------------------------
# БИБЛИОТЕКИ сторонние
#------------------------------------------

#------------------------------------------
# БИБЛИОТЕКИ LU
#------------------------------------------

"""
function LoadDLL (NameDll: string; var HandleDLL: HModule): Boolean;
{ LoadDLL }
begin
    HandleDLL := LoadLibrary (PChar(NameDll));
    Result := (HandleDLL >= HINSTANCE_ERROR);
end;

function UnLoadDLL (HandleDLL: HModule): Boolean;
{ UnLoadDLL }
begin
    Result := FreeLibrary (HandleDLL);
end;

function GetFunc (HandleDLL: HModule; NameFunc: string; var AddFunc: Pointer): Boolean;
{ GetFunc }
begin
    try
        AddFunc := GetProcAddress (HandleDLL, PChar(NameFunc));
    except
        AddFunc := nil;
    end;
    Result := Assigned (AddFunc);
end;

function ErrorString (Error: DWORD): string;
{ ErrorString }
begin
    Result := SysErrorMessage (Error);
end;

function LastErrorString: string;
{ LastErrorString }
begin
    Result := ErrorString (GetLastError);
end;
"""

#-------------------------------------------------
# IsTerminal
# Возвращает True, если текущая консоль является терминалом
#-------------------------------------------------
def IsTerminal () -> bool:
    """IsTerminal"""
#beginfunction
    return sys.stdout.isatty ()
#endfunction

#-------------------------------------------------
# TupleToStr
# Возвращает кортеж в виде строки, состоящей из элементов ATuple
#-------------------------------------------------
def TupleToStr (ATuple:()) -> str:
    """TupleToStr"""
#beginfunction
    LResult = ''
    i = 0
    j = 0
    if type(ATuple) is tuple:
        for r in ATuple:
            i = i + 1
            if r != '':
                j = j + 1
                if j == 1:
                    LResult = LResult + r
                else:
                    LResult = LResult + ';' + r
                #endif
            #endif
        #endfor
    else:
        if len (ATuple) > 0:
            LResult = LResult + ATuple
        #endif
    #endif
    return LResult
#endfunction

#---------------------------------------------------------
# main
#---------------------------------------------------------
def main ():
#beginfunction
    ...
#endfunction

#---------------------------------------------------------
#
#---------------------------------------------------------
#beginmodule
if __name__ == "__main__":
    main()
#endif

#endmodule
